@@ -1,13 +1,13 @@
-'''
+"""
 Metacells single-cell RNA sequencing.
-'''
+"""
 
-from .version import version as __version__
-__license__ = 'MIT'
-__author__ = 'Oren Ben-Kiki'
-__email__ = 'oren@ben-kiki.org'
+__author__ = "Oren Ben-Kiki"
+__email__ = "oren@ben-kiki.org"
+__version__ = "0.7.0"
 
 from .should_check_avx2 import SHOULD_CHECK_AVX2
+
 if SHOULD_CHECK_AVX2:
     from . import check_avx2
 
