@@ -1,26 +1,27 @@
-'''
+"""
 Candidates
 ----------
-'''
+"""
 
-import sys
-from math import ceil, floor
-from typing import Optional, Set, Tuple, Union
+from math import ceil
+from math import floor
+from typing import Optional
+from typing import Set
+from typing import Tuple
+from typing import Union
 
 import numpy as np
-from anndata import AnnData
+from anndata import AnnData  # type: ignore
 
+import metacells.extensions as xt  # type: ignore
 import metacells.parameters as pr
 import metacells.utilities as ut
 
-if not 'sphinx' in sys.argv[0]:
-    import metacells.extensions as xt  # type: ignore
-
 __all__ = [
-    'compute_candidate_metacells',
-    'choose_seeds',
-    'optimize_partitions',
-    'score_partitions',
+    "compute_candidate_metacells",
+    "choose_seeds",
+    "optimize_partitions",
+    "score_partitions",
 ]
 
 
@@ -29,7 +30,7 @@ __all__ = [
 @ut.expand_doc()
 def compute_candidate_metacells(  # pylint: disable=too-many-statements,too-many-branches
     adata: AnnData,
-    what: Union[str, ut.Matrix] = 'obs_outgoing_weights',
+    what: Union[str, ut.Matrix] = "obs_outgoing_weights",
     *,
     target_metacell_size: float,
     cell_sizes: Optional[Union[str, ut.Vector]] = pr.candidates_cell_sizes,
@@ -48,7 +49,7 @@ def compute_candidate_metacells(  # pylint: disable=too-many-statements,too-many
     random_seed: int = 0,
     inplace: bool = True,
 ) -> Optional[ut.PandasSeries]:
-    '''
+    """
     Assign observations (cells) to (raw, candidate) metacells based on ``what`` data. (a weighted
     directed graph).
 
@@ -108,8 +109,8 @@ def compute_candidate_metacells(  # pylint: disable=too-many-statements,too-many
        metacell). We again apply the ``cooldown_phase`` every time we re-optimize.
 
     6. Repeat the above steps until all metacells candidates are in the acceptable size range.
-    '''
-    edge_weights = ut.get_oo_proper(adata, what, layout='row_major')
+    """
+    edge_weights = ut.get_oo_proper(adata, what, layout="row_major")
     assert edge_weights.shape[0] == edge_weights.shape[1]
     assert 0.0 < cooldown_pass < 1.0
     assert 0.0 <= cooldown_node <= 1.0
@@ -119,26 +120,23 @@ def compute_candidate_metacells(  # pylint: disable=too-many-statements,too-many
 
     outgoing_edge_weights = ut.mustbe_compressed_matrix(edge_weights)
 
-    assert ut.matrix_layout(outgoing_edge_weights) == 'row_major'
-    incoming_edge_weights = \
-        ut.mustbe_compressed_matrix(ut.to_layout(outgoing_edge_weights,
-                                                 layout='column_major'))
-    assert ut.matrix_layout(incoming_edge_weights) == 'column_major'
+    assert ut.matrix_layout(outgoing_edge_weights) == "row_major"
+    incoming_edge_weights = ut.mustbe_compressed_matrix(ut.to_layout(outgoing_edge_weights, layout="column_major"))
+    assert ut.matrix_layout(incoming_edge_weights) == "column_major"
 
-    assert outgoing_edge_weights.data.dtype == 'float32'
-    assert outgoing_edge_weights.indices.dtype == 'int32'
-    assert outgoing_edge_weights.indptr.dtype == 'int32'
-    assert incoming_edge_weights.data.dtype == 'float32'
-    assert incoming_edge_weights.indices.dtype == 'int32'
-    assert incoming_edge_weights.indptr.dtype == 'int32'
+    assert outgoing_edge_weights.data.dtype == "float32"
+    assert outgoing_edge_weights.indices.dtype == "int32"
+    assert outgoing_edge_weights.indptr.dtype == "int32"
+    assert incoming_edge_weights.data.dtype == "float32"
+    assert incoming_edge_weights.indices.dtype == "int32"
+    assert incoming_edge_weights.indptr.dtype == "int32"
 
-    node_sizes = \
-        ut.maybe_o_numpy(adata, cell_sizes, formatter=ut.sizes_description)
+    node_sizes = ut.maybe_o_numpy(adata, cell_sizes, formatter=ut.sizes_description)
     if node_sizes is None:
-        node_sizes = np.full(size, 1.0, dtype='float32')
+        node_sizes = np.full(size, 1.0, dtype="float32")
     else:
-        node_sizes = node_sizes.astype('float32')
-    ut.log_calc('node_sizes', node_sizes, formatter=ut.sizes_description)
+        node_sizes = node_sizes.astype("float32")
+    ut.log_calc("node_sizes", node_sizes, formatter=ut.sizes_description)
 
     assert target_metacell_size > 0
     max_metacell_size = None
@@ -146,19 +144,19 @@ def compute_candidate_metacells(  # pylint: disable=too-many-statements,too-many
 
     if min_split_size_factor is not None:
         assert min_split_size_factor > 0
-        max_metacell_size = \
-            ceil(target_metacell_size * min_split_size_factor) - 1
-    ut.log_calc('max_metacell_size', max_metacell_size)
+        max_metacell_size = ceil(target_metacell_size * min_split_size_factor) - 1
+    ut.log_calc("max_metacell_size", max_metacell_size)
 
     if max_merge_size_factor is not None:
         assert max_merge_size_factor > 0
-        min_metacell_size = \
-            floor(target_metacell_size * max_merge_size_factor) + 1
-    ut.log_calc('min_metacell_size', min_metacell_size)
+        min_metacell_size = floor(target_metacell_size * max_merge_size_factor) + 1
+    ut.log_calc("min_metacell_size", min_metacell_size)
 
-    target_metacell_cells = max(1.0 if min_metacell_cells is None else float(min_metacell_cells),
-                                float(target_metacell_size / np.mean(node_sizes)))
-    ut.log_calc('target_metacell_cells', target_metacell_cells)
+    target_metacell_cells = max(
+        1.0 if min_metacell_cells is None else float(min_metacell_cells),
+        float(target_metacell_size / np.mean(node_sizes)),
+    )
+    ut.log_calc("target_metacell_cells", target_metacell_cells)
 
     if min_split_size_factor is not None and max_merge_size_factor is not None:
         assert max_merge_size_factor < min_split_size_factor
@@ -166,26 +164,26 @@ def compute_candidate_metacells(  # pylint: disable=too-many-statements,too-many
         assert max_metacell_size is not None
         assert min_metacell_size <= max_metacell_size
 
-    community_of_nodes = \
-        ut.maybe_o_numpy(adata, cell_seeds, formatter=ut.groups_description)
+    community_of_nodes = ut.maybe_o_numpy(adata, cell_seeds, formatter=ut.groups_description)
 
     if community_of_nodes is not None:
-        assert community_of_nodes.dtype == 'int32'
+        assert community_of_nodes.dtype == "int32"
     else:
         target_seeds_count = ceil(size / target_metacell_cells)
-        ut.log_calc('target_seeds_count', target_seeds_count)
+        ut.log_calc("target_seeds_count", target_seeds_count)
 
-        community_of_nodes = np.full(size, -1, dtype='int32')
-        _choose_seeds(outgoing_edge_weights=outgoing_edge_weights,
-                      incoming_edge_weights=incoming_edge_weights,
-                      seed_of_cells=community_of_nodes,
-                      max_seeds_count=target_seeds_count,
-                      min_seed_size_quantile=min_seed_size_quantile,
-                      max_seed_size_quantile=max_seed_size_quantile,
-                      random_seed=random_seed)
+        community_of_nodes = np.full(size, -1, dtype="int32")
+        _choose_seeds(
+            outgoing_edge_weights=outgoing_edge_weights,
+            incoming_edge_weights=incoming_edge_weights,
+            seed_of_cells=community_of_nodes,
+            max_seeds_count=target_seeds_count,
+            min_seed_size_quantile=min_seed_size_quantile,
+            max_seed_size_quantile=max_seed_size_quantile,
+            random_seed=random_seed,
+        )
 
-    ut.set_o_data(adata, 'seed', community_of_nodes,
-                  formatter=ut.groups_description)
+    ut.set_o_data(adata, "seed", community_of_nodes, formatter=ut.groups_description)
     community_of_nodes = community_of_nodes.copy()
 
     np.random.seed(random_seed)
@@ -199,65 +197,66 @@ def compute_candidate_metacells(  # pylint: disable=too-many-statements,too-many
     kept_communities_count = 0
 
     while True:
-        cold_temperature, score = \
-            _optimize_split_communities(  #
-                outgoing_edge_weights=outgoing_edge_weights,
-                incoming_edge_weights=incoming_edge_weights,
-                community_of_nodes=community_of_nodes,
-                node_sizes=node_sizes,
-                target_metacell_size=target_metacell_size,
-                max_metacell_size=max_metacell_size,
-                max_split_min_cut_strength=max_split_min_cut_strength,
-                min_cut_seed_cells=min_cut_seed_cells,
-                must_complete_cover=must_complete_cover,
-                min_seed_size_quantile=min_seed_size_quantile,
-                max_seed_size_quantile=max_seed_size_quantile,
-                random_seed=random_seed,
-                cooldown_pass=cooldown_pass,
-                cooldown_node=cooldown_node,
-                cooldown_phase=cooldown_phase,
-                kept_communities_count=kept_communities_count,
-                cold_temperature=cold_temperature,
-                atomic_candidates=atomic_candidates)
+        cold_temperature, score = _optimize_split_communities(  #
+            outgoing_edge_weights=outgoing_edge_weights,
+            incoming_edge_weights=incoming_edge_weights,
+            community_of_nodes=community_of_nodes,
+            node_sizes=node_sizes,
+            target_metacell_size=target_metacell_size,
+            max_metacell_size=max_metacell_size,
+            max_split_min_cut_strength=max_split_min_cut_strength,
+            min_cut_seed_cells=min_cut_seed_cells,
+            must_complete_cover=must_complete_cover,
+            min_seed_size_quantile=min_seed_size_quantile,
+            max_seed_size_quantile=max_seed_size_quantile,
+            random_seed=random_seed,
+            cooldown_pass=cooldown_pass,
+            cooldown_node=cooldown_node,
+            cooldown_phase=cooldown_phase,
+            kept_communities_count=kept_communities_count,
+            cold_temperature=cold_temperature,
+            atomic_candidates=atomic_candidates,
+        )
 
-        small_communities, small_nodes_count = \
-            _find_small_communities(community_of_nodes=community_of_nodes,
-                                    node_sizes=node_sizes,
-                                    min_metacell_size=min_metacell_size,
-                                    min_metacell_cells=min_metacell_cells)
+        small_communities, small_nodes_count = _find_small_communities(
+            community_of_nodes=community_of_nodes,
+            node_sizes=node_sizes,
+            min_metacell_size=min_metacell_size,
+            min_metacell_cells=min_metacell_cells,
+        )
 
         small_communities_count = len(small_communities)
         if small_communities_count < 2:
             break
 
         if (old_small_nodes_count, old_score) <= (small_nodes_count, score):
-            ut.logger().debug('is not better, revert')
+            ut.logger().debug("is not better, revert")
             community_of_nodes = old_communities
             score = old_score
-            ut.log_calc('communities', community_of_nodes,
-                        formatter=ut.groups_description)
-            ut.log_calc('score', score)
+            ut.log_calc("communities", community_of_nodes, formatter=ut.groups_description)
+            ut.log_calc("score", score)
             break
 
         old_score = score
         old_communities = community_of_nodes.copy()
         old_small_nodes_count = small_nodes_count
 
-        kept_communities_count = \
-            _cancel_communities(community_of_nodes=community_of_nodes,
-                                cancelled_communities=small_communities)
+        kept_communities_count = _cancel_communities(
+            community_of_nodes=community_of_nodes, cancelled_communities=small_communities
+        )
 
-        _choose_seeds(outgoing_edge_weights=outgoing_edge_weights,
-                      incoming_edge_weights=incoming_edge_weights,
-                      seed_of_cells=community_of_nodes,
-                      max_seeds_count=kept_communities_count + small_communities_count - 1,
-                      min_seed_size_quantile=min_seed_size_quantile,
-                      max_seed_size_quantile=max_seed_size_quantile,
-                      random_seed=random_seed)
+        _choose_seeds(
+            outgoing_edge_weights=outgoing_edge_weights,
+            incoming_edge_weights=incoming_edge_weights,
+            seed_of_cells=community_of_nodes,
+            max_seeds_count=kept_communities_count + small_communities_count - 1,
+            min_seed_size_quantile=min_seed_size_quantile,
+            max_seed_size_quantile=max_seed_size_quantile,
+            random_seed=random_seed,
+        )
 
     if inplace:
-        ut.set_o_data(adata, 'candidate', community_of_nodes,
-                      formatter=ut.groups_description)
+        ut.set_o_data(adata, "candidate", community_of_nodes, formatter=ut.groups_description)
         return None
 
     if must_complete_cover:
@@ -265,8 +264,7 @@ def compute_candidate_metacells(  # pylint: disable=too-many-statements,too-many
     else:
         community_of_nodes[community_of_nodes < 0] = -1
 
-    ut.log_return('candidate', community_of_nodes,
-                  formatter=ut.groups_description)
+    ut.log_return("candidate", community_of_nodes, formatter=ut.groups_description)
     return ut.to_pandas_series(community_of_nodes, index=adata.obs_names)
 
 
@@ -291,43 +289,44 @@ def _optimize_split_communities(
     atomic_candidates: Set[Tuple[int, ...]],
 ) -> Tuple[float, float]:
     while True:
-        ut.log_calc('cold_temperature', cold_temperature)
-        score = \
-            _optimize_partitions(outgoing_edge_weights=outgoing_edge_weights,
-                                 incoming_edge_weights=incoming_edge_weights,
-                                 community_of_nodes=community_of_nodes,
-                                 random_seed=random_seed,
-                                 cooldown_pass=cooldown_pass,
-                                 cooldown_node=cooldown_node,
-                                 cold_communities_count=kept_communities_count,
-                                 cold_temperature=cold_temperature)
+        ut.log_calc("cold_temperature", cold_temperature)
+        score = _optimize_partitions(
+            outgoing_edge_weights=outgoing_edge_weights,
+            incoming_edge_weights=incoming_edge_weights,
+            community_of_nodes=community_of_nodes,
+            random_seed=random_seed,
+            cooldown_pass=cooldown_pass,
+            cooldown_node=cooldown_node,
+            cold_communities_count=kept_communities_count,
+            cold_temperature=cold_temperature,
+        )
 
         cold_temperature = cold_temperature * (1 - cooldown_phase)
-        ut.log_calc('communities', community_of_nodes,
-                    formatter=ut.groups_description)
+        ut.log_calc("communities", community_of_nodes, formatter=ut.groups_description)
 
-        modified_communities_count, \
-            new_communities_count, \
-            kept_communities_count = \
-            _split_communities(outgoing_edge_weights=outgoing_edge_weights,
-                               community_of_nodes=community_of_nodes,
-                               node_sizes=node_sizes,
-                               target_metacell_size=target_metacell_size,
-                               max_metacell_size=max_metacell_size,
-                               max_split_min_cut_strength=max_split_min_cut_strength,
-                               min_cut_seed_cells=min_cut_seed_cells,
-                               must_complete_cover=must_complete_cover,
-                               atomic_candidates=atomic_candidates)
+        modified_communities_count, new_communities_count, kept_communities_count = _split_communities(
+            outgoing_edge_weights=outgoing_edge_weights,
+            community_of_nodes=community_of_nodes,
+            node_sizes=node_sizes,
+            target_metacell_size=target_metacell_size,
+            max_metacell_size=max_metacell_size,
+            max_split_min_cut_strength=max_split_min_cut_strength,
+            min_cut_seed_cells=min_cut_seed_cells,
+            must_complete_cover=must_complete_cover,
+            atomic_candidates=atomic_candidates,
+        )
 
         if new_communities_count > 0:
             assert modified_communities_count > 0
-            _choose_seeds(outgoing_edge_weights=outgoing_edge_weights,
-                          incoming_edge_weights=incoming_edge_weights,
-                          seed_of_cells=community_of_nodes,
-                          max_seeds_count=kept_communities_count + new_communities_count,
-                          min_seed_size_quantile=min_seed_size_quantile,
-                          max_seed_size_quantile=max_seed_size_quantile,
-                          random_seed=random_seed)
+            _choose_seeds(
+                outgoing_edge_weights=outgoing_edge_weights,
+                incoming_edge_weights=incoming_edge_weights,
+                seed_of_cells=community_of_nodes,
+                max_seeds_count=kept_communities_count + new_communities_count,
+                min_seed_size_quantile=min_seed_size_quantile,
+                max_seed_size_quantile=max_seed_size_quantile,
+                random_seed=random_seed,
+            )
 
         if modified_communities_count == 0:
             return cold_temperature, score
@@ -348,9 +347,8 @@ def _split_communities(  # pylint: disable=too-many-statements
     communities_count = np.max(community_of_nodes) + 1
     assert communities_count > 0
 
-    if max_metacell_size is None \
-            and max_split_min_cut_strength is None:
-        ut.logger().debug('no communities need to be split')
+    if max_metacell_size is None and max_split_min_cut_strength is None:
+        ut.logger().debug("no communities need to be split")
         return (0, 0, communities_count)
 
     cancelled_communities: Set[int] = set()
@@ -365,8 +363,9 @@ def _split_communities(  # pylint: disable=too-many-statements
         community_size = np.sum(node_sizes[community_mask])
 
         if max_metacell_size is not None and community_size > max_metacell_size:
-            ut.logger().debug('community: %s nodes: %s size: %s is too large',
-                              community_index, np.sum(community_mask), community_size)
+            ut.logger().debug(
+                "community: %s nodes: %s size: %s is too large", community_index, np.sum(community_mask), community_size
+            )
             cancelled_total_size += community_size
             cancelled_communities.add(community_index)
             continue
@@ -378,18 +377,23 @@ def _split_communities(  # pylint: disable=too-many-statements
         if community_indices in atomic_candidates:
             continue
 
-        did_cut, did_split = \
-            _min_cut_community(outgoing_edge_weights=outgoing_edge_weights,
-                               community_of_nodes=community_of_nodes,
-                               cut_community_index=community_index,
-                               max_split_min_cut_strength=max_split_min_cut_strength,
-                               min_cut_seed_cells=min_cut_seed_cells,
-                               must_complete_cover=must_complete_cover,
-                               new_community_index=next_new_community_index)
+        did_cut, did_split = _min_cut_community(
+            outgoing_edge_weights=outgoing_edge_weights,
+            community_of_nodes=community_of_nodes,
+            cut_community_index=community_index,
+            max_split_min_cut_strength=max_split_min_cut_strength,
+            min_cut_seed_cells=min_cut_seed_cells,
+            must_complete_cover=must_complete_cover,
+            new_community_index=next_new_community_index,
+        )
         if did_cut:
             cut_communities_count += 1
-            ut.logger().debug('community: %s nodes: %s size: %s is too divided',
-                              community_index, np.sum(community_mask), community_size)
+            ut.logger().debug(
+                "community: %s nodes: %s size: %s is too divided",
+                community_index,
+                np.sum(community_mask),
+                community_size,
+            )
             if did_split:
                 split_communities_count += 1
                 next_new_community_index += 1
@@ -400,31 +404,27 @@ def _split_communities(  # pylint: disable=too-many-statements
     cancelled_communities_count = len(cancelled_communities)
     modified_communities_count = cut_communities_count + cancelled_communities_count
     if modified_communities_count == 0:
-        ut.logger().debug('no communities were modified')
+        ut.logger().debug("no communities were modified")
         return (0, 0, communities_count)
 
-    ut.logger().debug('all communities: %s', communities_count)
-    ut.logger().debug('cut communities: %s', cut_communities_count)
-    ut.logger().debug('split communities: %s', split_communities_count)
-    ut.logger().debug('cancelled communities: %s', cancelled_communities_count)
+    ut.logger().debug("all communities: %s", communities_count)
+    ut.logger().debug("cut communities: %s", cut_communities_count)
+    ut.logger().debug("split communities: %s", split_communities_count)
+    ut.logger().debug("cancelled communities: %s", cancelled_communities_count)
 
     if cancelled_communities_count > 0:
-        new_communities_count = \
-            max(ceil(cancelled_total_size / target_metacell_size),
-                cancelled_communities_count + 1)
+        new_communities_count = max(ceil(cancelled_total_size / target_metacell_size), cancelled_communities_count + 1)
     else:
         new_communities_count = 0
 
     if did_not_add and new_communities_count == 0:
         new_communities_count = 1
 
-    ut.logger().debug('reclaimed communities: %s', new_communities_count)
+    ut.logger().debug("reclaimed communities: %s", new_communities_count)
 
-    kept_communities_count = \
-        _cancel_communities(community_of_nodes, cancelled_communities)
-    ut.logger().debug('kept communities: %s', kept_communities_count)
-    assert kept_communities_count == \
-        communities_count + split_communities_count - cancelled_communities_count
+    kept_communities_count = _cancel_communities(community_of_nodes, cancelled_communities)
+    ut.logger().debug("kept communities: %s", kept_communities_count)
+    assert kept_communities_count == communities_count + split_communities_count - cancelled_communities_count
 
     return (modified_communities_count, new_communities_count, kept_communities_count)
 
@@ -436,23 +436,26 @@ def _min_cut_community(
     max_split_min_cut_strength: float,
     min_cut_seed_cells: Optional[int],
     must_complete_cover: bool,
-    new_community_index: int
+    new_community_index: int,
 ) -> Tuple[bool, bool]:
     community_mask = community_of_nodes == cut_community_index
-    community_edge_weights = outgoing_edge_weights[community_mask,
-                                                   :][:, community_mask]
+    if np.sum(community_mask) < 2:
+        return (False, False)
+    community_edge_weights = outgoing_edge_weights[community_mask, :][:, community_mask]
     community_edge_weights += community_edge_weights.T
 
     cut, cut_strength = ut.min_cut(community_edge_weights)
     if cut_strength is None:
         return (False, False)
 
-    ut.logger().debug('min cut community: %s partitions: %s + %s = %s strength: %s',
-                      cut_community_index,
-                      len(cut.partition[0]),
-                      len(cut.partition[1]),
-                      community_edge_weights.shape[0],
-                      cut_strength)
+    ut.logger().debug(
+        "min cut community: %s partitions: %s + %s = %s strength: %s",
+        cut_community_index,
+        len(cut.partition[0]),
+        len(cut.partition[1]),
+        community_edge_weights.shape[0],
+        cut_strength,
+    )
 
     if cut_strength > max_split_min_cut_strength:
         return (False, False)
@@ -470,8 +473,7 @@ def _min_cut_community(
                 return (False, False)
             small_community_indices = community_indices[cut.partition[small_partition]]
             community_of_nodes[small_community_indices] = -2
-            ut.logger().debug('give up on small cut: %s',
-                              len(cut.partition[small_partition]))
+            ut.logger().debug("give up on small cut: %s", len(cut.partition[small_partition]))
             return (True, False)
 
     second_partition_indices = community_indices[cut.partition[1]]
@@ -489,9 +491,8 @@ def _find_small_communities(
     communities_count = np.max(community_of_nodes) + 1
     assert communities_count > 0
 
-    if min_metacell_size is None \
-            and min_metacell_cells is None:
-        ut.logger().debug('all communities are not too small')
+    if min_metacell_size is None and min_metacell_cells is None:
+        ut.logger().debug("all communities are not too small")
         return (set(), communities_count)
 
     small_communities: Set[int] = set()
@@ -503,11 +504,11 @@ def _find_small_communities(
         community_size = np.sum(node_sizes[community_mask])
 
         if min_metacell_cells is not None and community_nodes_count < min_metacell_cells:
-            ut.logger().debug('community: %s nodes: %s is too few',
-                              community_index, community_nodes_count)
+            ut.logger().debug("community: %s nodes: %s is too few", community_index, community_nodes_count)
         elif min_metacell_size is not None and community_size < min_metacell_size:
-            ut.logger().debug('community: %s nodes: %s size: %s is too small',
-                              community_index, community_nodes_count, community_size)
+            ut.logger().debug(
+                "community: %s nodes: %s size: %s is too small", community_index, community_nodes_count, community_size
+            )
         else:
             continue
 
@@ -517,10 +518,7 @@ def _find_small_communities(
     return (small_communities, small_nodes_count)
 
 
-def _cancel_communities(
-    community_of_nodes: ut.NumpyVector,
-    cancelled_communities: Set[int]
-) -> int:
+def _cancel_communities(community_of_nodes: ut.NumpyVector, cancelled_communities: Set[int]) -> int:
     communities_count = np.max(community_of_nodes) + 1
     kept_communities_count = 0
     for community_index in range(communities_count):
@@ -529,13 +527,11 @@ def _cancel_communities(
             continue
 
         if community_index > kept_communities_count:
-            community_of_nodes[community_of_nodes ==
-                               community_index] = kept_communities_count
+            community_of_nodes[community_of_nodes == community_index] = kept_communities_count
 
         kept_communities_count += 1
 
-    assert kept_communities_count == communities_count - \
-        len(cancelled_communities)
+    assert kept_communities_count == communities_count - len(cancelled_communities)
     return kept_communities_count
 
 
@@ -550,7 +546,7 @@ def choose_seeds(
     max_seed_size_quantile: float = pr.max_seed_size_quantile,
     random_seed: int,
 ) -> ut.NumpyVector:
-    '''
+    """
     Choose initial assignment of cells to seeds based on the ``edge_weights``.
 
     Returns a vector assigning each node (cell) to a seed (initial community).
@@ -577,33 +573,31 @@ def choose_seeds(
        connected nodes of the remaining unassigned nodes.
 
     4. We repeat this until we reach the target number of seeds.
-    '''
+    """
     size = edge_weights.shape[0]
 
     outgoing_edge_weights = ut.mustbe_compressed_matrix(edge_weights)
-    assert ut.matrix_layout(outgoing_edge_weights) == 'row_major'
+    assert ut.matrix_layout(outgoing_edge_weights) == "row_major"
 
-    incoming_edge_weights = \
-        ut.mustbe_compressed_matrix(ut.to_layout(outgoing_edge_weights,
-                                                 layout='column_major'))
-    assert ut.matrix_layout(incoming_edge_weights) == 'column_major'
+    incoming_edge_weights = ut.mustbe_compressed_matrix(ut.to_layout(outgoing_edge_weights, layout="column_major"))
+    assert ut.matrix_layout(incoming_edge_weights) == "column_major"
 
     if seed_of_cells is None:
-        seed_of_cells = np.full(size, -1, dtype='int32')
+        seed_of_cells = np.full(size, -1, dtype="int32")
     else:
-        assert seed_of_cells.dtype == 'int32'
+        assert seed_of_cells.dtype == "int32"
 
-    assert outgoing_edge_weights.shape \
-        == incoming_edge_weights.shape \
-        == (len(seed_of_cells), len(seed_of_cells))
+    assert outgoing_edge_weights.shape == incoming_edge_weights.shape == (len(seed_of_cells), len(seed_of_cells))
 
-    return _choose_seeds(outgoing_edge_weights=outgoing_edge_weights,
-                         incoming_edge_weights=incoming_edge_weights,
-                         seed_of_cells=seed_of_cells,
-                         max_seeds_count=max_seeds_count,
-                         min_seed_size_quantile=min_seed_size_quantile,
-                         max_seed_size_quantile=max_seed_size_quantile,
-                         random_seed=random_seed)
+    return _choose_seeds(
+        outgoing_edge_weights=outgoing_edge_weights,
+        incoming_edge_weights=incoming_edge_weights,
+        seed_of_cells=seed_of_cells,
+        max_seeds_count=max_seeds_count,
+        min_seed_size_quantile=min_seed_size_quantile,
+        max_seed_size_quantile=max_seed_size_quantile,
+        random_seed=random_seed,
+    )
 
 
 @ut.timed_call()
@@ -617,19 +611,21 @@ def _choose_seeds(
     max_seed_size_quantile: float,
     random_seed: int,
 ) -> ut.NumpyVector:
-    xt.choose_seeds(outgoing_edge_weights.data,
-                    outgoing_edge_weights.indices,
-                    outgoing_edge_weights.indptr,
-                    incoming_edge_weights.data,
-                    incoming_edge_weights.indices,
-                    incoming_edge_weights.indptr,
-                    random_seed,
-                    max_seeds_count,
-                    min_seed_size_quantile,
-                    max_seed_size_quantile,
-                    seed_of_cells)
+    xt.choose_seeds(
+        outgoing_edge_weights.data,
+        outgoing_edge_weights.indices,
+        outgoing_edge_weights.indptr,
+        incoming_edge_weights.data,
+        incoming_edge_weights.indices,
+        incoming_edge_weights.indptr,
+        random_seed,
+        max_seeds_count,
+        min_seed_size_quantile,
+        max_seed_size_quantile,
+        seed_of_cells,
+    )
 
-    ut.log_calc('seeds', seed_of_cells, formatter=ut.groups_description)
+    ut.log_calc("seeds", seed_of_cells, formatter=ut.groups_description)
     return seed_of_cells
 
 
@@ -643,7 +639,7 @@ def optimize_partitions(
     cooldown_node: float = pr.cooldown_node,
     random_seed: int,
 ) -> float:
-    '''
+    """
     Optimize partition to candidate metacells (communities) using the ``edge_weights``.
 
     Returns the score of the optimized partition.
@@ -674,22 +670,22 @@ def optimize_partitions(
 
     This simulated-annealing-like behavior helps the algorithm to escape local maximums, although of
     course no claim is made of achieving the global maximum of the goal function.
-    '''
+    """
     outgoing_edge_weights = ut.mustbe_compressed_matrix(edge_weights)
-    assert ut.matrix_layout(outgoing_edge_weights) == 'row_major'
+    assert ut.matrix_layout(outgoing_edge_weights) == "row_major"
 
-    incoming_edge_weights = \
-        ut.mustbe_compressed_matrix(ut.to_layout(outgoing_edge_weights,
-                                                 layout='column_major'))
-    assert ut.matrix_layout(incoming_edge_weights) == 'column_major'
-    return _optimize_partitions(outgoing_edge_weights=outgoing_edge_weights,
-                                incoming_edge_weights=incoming_edge_weights,
-                                random_seed=random_seed,
-                                cooldown_pass=cooldown_pass,
-                                cooldown_node=cooldown_node,
-                                community_of_nodes=community_of_nodes,
-                                cold_communities_count=0,
-                                cold_temperature=cooldown_pass)
+    incoming_edge_weights = ut.mustbe_compressed_matrix(ut.to_layout(outgoing_edge_weights, layout="column_major"))
+    assert ut.matrix_layout(incoming_edge_weights) == "column_major"
+    return _optimize_partitions(
+        outgoing_edge_weights=outgoing_edge_weights,
+        incoming_edge_weights=incoming_edge_weights,
+        random_seed=random_seed,
+        cooldown_pass=cooldown_pass,
+        cooldown_node=cooldown_node,
+        community_of_nodes=community_of_nodes,
+        cold_communities_count=0,
+        cold_temperature=cooldown_pass,
+    )
 
 
 @ut.timed_call()
@@ -704,20 +700,22 @@ def _optimize_partitions(
     cold_temperature: float,
     random_seed: int,
 ) -> float:
-    assert community_of_nodes.dtype == 'int32'
-    score = xt.optimize_partitions(outgoing_edge_weights.data,
-                                   outgoing_edge_weights.indices,
-                                   outgoing_edge_weights.indptr,
-                                   incoming_edge_weights.data,
-                                   incoming_edge_weights.indices,
-                                   incoming_edge_weights.indptr,
-                                   random_seed,
-                                   cooldown_pass,
-                                   cooldown_node,
-                                   community_of_nodes,
-                                   cold_communities_count,
-                                   cold_temperature)
-    ut.log_calc('score', score)
+    assert community_of_nodes.dtype == "int32"
+    score = xt.optimize_partitions(
+        outgoing_edge_weights.data,
+        outgoing_edge_weights.indices,
+        outgoing_edge_weights.indptr,
+        incoming_edge_weights.data,
+        incoming_edge_weights.indices,
+        incoming_edge_weights.indptr,
+        random_seed,
+        cooldown_pass,
+        cooldown_node,
+        community_of_nodes,
+        cold_communities_count,
+        cold_temperature,
+    )
+    ut.log_calc("score", score)
     return score
 
 
@@ -729,7 +727,7 @@ def score_partitions(
     partition_of_nodes: ut.NumpyVector,
     with_orphans: bool = True,
 ) -> None:
-    '''
+    """
     Compute the "stability" the "stability" goal function which is defined to be the ratio between
     (1) the probability that, selecting a random node and either a random outgoing edge or a random
     incoming edge (biased by their weights), that the node connected to by that edge is in the same
@@ -740,26 +738,26 @@ def score_partitions(
     general we add 1e-6 to the product of the incoming and outgoing weights so we can safely log it
     for efficient computation; thus orphans are given a very small (non-zero) weight so the overall
     score is not zeroed even when including them.
-    '''
-    assert str(partition_of_nodes.dtype) == 'int32'
+    """
+    assert str(partition_of_nodes.dtype) == "int32"
     outgoing_edge_weights = ut.mustbe_compressed_matrix(edge_weights)
-    assert ut.matrix_layout(outgoing_edge_weights) == 'row_major'
+    assert ut.matrix_layout(outgoing_edge_weights) == "row_major"
 
-    incoming_edge_weights = \
-        ut.mustbe_compressed_matrix(ut.to_layout(outgoing_edge_weights,
-                                                 layout='column_major'))
-    assert ut.matrix_layout(incoming_edge_weights) == 'column_major'
+    incoming_edge_weights = ut.mustbe_compressed_matrix(ut.to_layout(outgoing_edge_weights, layout="column_major"))
+    assert ut.matrix_layout(incoming_edge_weights) == "column_major"
 
     with ut.unfrozen(partition_of_nodes):
-        with ut.timed_step('.score'):
-            score = xt.score_partitions(outgoing_edge_weights.data,
-                                        outgoing_edge_weights.indices,
-                                        outgoing_edge_weights.indptr,
-                                        incoming_edge_weights.data,
-                                        incoming_edge_weights.indices,
-                                        incoming_edge_weights.indptr,
-                                        partition_of_nodes,
-                                        with_orphans)
+        with ut.timed_step(".score"):
+            score = xt.score_partitions(
+                outgoing_edge_weights.data,
+                outgoing_edge_weights.indices,
+                outgoing_edge_weights.indptr,
+                incoming_edge_weights.data,
+                incoming_edge_weights.indices,
+                incoming_edge_weights.indptr,
+                partition_of_nodes,
+                with_orphans,
+            )
 
-    ut.log_calc('score', score)
+    ut.log_calc("score", score)
     return score
