@@ -1,16 +1,17 @@
-'''
+"""
 Mask
 ----
-'''
+"""
 
-from typing import List, Optional
+from typing import List
+from typing import Optional
 
-from anndata import AnnData
+from anndata import AnnData  # type: ignore
 
 import metacells.utilities as ut
 
 __all__ = [
-    'combine_masks',
+    "combine_masks",
 ]
 
 
@@ -24,7 +25,7 @@ def combine_masks(  # pylint: disable=too-many-branches,too-many-statements
     invert: bool = False,
     to: Optional[str] = None,
 ) -> Optional[ut.PandasSeries]:
-    '''
+    """
     Combine different pre-computed masks into a final overall mask.
 
     **Input**
@@ -46,7 +47,7 @@ def combine_masks(  # pylint: disable=too-many-branches,too-many-statements
     2. Combine (bitwise-OR) the AND mask and the OR mask into a single mask.
 
     3. If ``invert`` (default: {invert}), invert the result combined mask.
-    '''
+    """
     assert len(masks) > 0
 
     per: Optional[str] = None
@@ -57,39 +58,37 @@ def combine_masks(  # pylint: disable=too-many-branches,too-many-statements
     for mask_name in masks:
         log_mask_name = mask_name
 
-        if mask_name[0] == '|':
+        if mask_name[0] == "|":
             is_or = True
             mask_name = mask_name[1:]
         else:
             is_or = False
 
-        if mask_name[0] == '~':
+        if mask_name[0] == "~":
             invert_mask = True
             mask_name = mask_name[1:]
         else:
             invert_mask = False
 
-        if mask_name[-1] == '?':
+        if mask_name[-1] == "?":
             must_exist = False
             mask_name = mask_name[:-1]
         else:
             must_exist = True
 
         if mask_name in adata.obs:
-            mask_per = 'o'
-            mask = ut.get_o_numpy(adata, mask_name,
-                                  formatter=ut.mask_description) > 0
+            mask_per = "o"
+            mask = ut.get_o_numpy(adata, mask_name, formatter=ut.mask_description) > 0
         elif mask_name in adata.var:
-            mask_per = 'v'
-            mask = ut.get_v_numpy(adata, mask_name,
-                                  formatter=ut.mask_description) > 0
+            mask_per = "v"
+            mask = ut.get_v_numpy(adata, mask_name, formatter=ut.mask_description) > 0
         else:
             if must_exist:
-                raise KeyError(f'unknown mask data: {mask_name}')
+                raise KeyError(f"unknown mask data: {mask_name}")
             continue
 
-        if mask.dtype != 'bool':
-            raise ValueError(f'the data: {mask_name} is not a boolean mask')
+        if mask.dtype != "bool":
+            raise ValueError(f"the data: {mask_name} is not a boolean mask")
 
         if invert_mask:
             mask = ~mask
@@ -101,8 +100,7 @@ def combine_masks(  # pylint: disable=too-many-branches,too-many-statements
             per = mask_per
         else:
             if mask_per != per:
-                raise \
-                    ValueError('mixing per-observation and per-variable masks')
+                raise ValueError("mixing per-observation and per-variable masks")
 
         if is_or:
             if or_mask is None:
@@ -124,19 +122,19 @@ def combine_masks(  # pylint: disable=too-many-branches,too-many-statements
         if or_mask is not None:
             combined_mask = or_mask
         else:
-            raise ValueError('no masks to combine')
+            raise ValueError("no masks to combine")
 
     if invert:
         combined_mask = ~combined_mask
 
     if to is None:
-        ut.log_return('combined', combined_mask)
-        if per == 'o':
+        ut.log_return("combined", combined_mask)
+        if per == "o":
             return ut.to_pandas_series(combined_mask, index=adata.obs_names)
-        assert per == 'v'
+        assert per == "v"
         return ut.to_pandas_series(combined_mask, index=adata.var_names)
 
-    if per == 'o':
+    if per == "o":
         ut.set_o_data(adata, to, combined_mask)
     else:
         ut.set_v_data(adata, to, combined_mask)
