@@ -1,15 +1,14 @@
-'''
+"""
 Generic utilities used by the metacells code.
 
 Arguably all(most) of these belong in more general package(s).
 
 All the functions included here are exported under ``metacells.ut``.
-'''
+"""
 
 from .annotation import *  # pylint: disable=redefined-builtin
 from .computation import *
 from .documentation import *
-from .hardware_info import *
 from .logging import *
 from .parallel import *
 from .timing import *
