@@ -1,4 +1,4 @@
-'''
+"""
 Annotation
 ----------
 
@@ -60,12 +60,19 @@ automated logging provided by the :py:mod:`metacells.utilities.logging` module.
 will automatically log writing the final results of a computation to the user at the ``INFO`` log
 level, while higher logging level give insight into the exact data being read and written by the
 algorithm's nested sub-steps.
-'''
+"""
 
-from typing import (Any, Callable, Collection, Dict, List, MutableMapping,
-                    Optional, Tuple, Union)
+from typing import Any
+from typing import Callable
+from typing import Collection
+from typing import Dict
+from typing import List
+from typing import MutableMapping
+from typing import Optional
+from typing import Tuple
+from typing import Union
 
-import anndata as ad
+import anndata as ad  # type: ignore
 import numpy as np
 from anndata import AnnData
 
@@ -75,48 +82,38 @@ import metacells.utilities.timing as utm
 import metacells.utilities.typing as utt
 
 __all__ = [
-    'slice',
-    'copy_adata',
-
-    'set_name',
-    'get_name',
-
-    'set_m_data',
-    'get_m_data',
-
-    'set_o_data',
-    'get_o_series',
-    'get_o_numpy',
-    'get_o_names',
-    'maybe_o_numpy',
-
-    'set_v_data',
-    'get_v_series',
-    'get_v_numpy',
-    'get_v_names',
-    'maybe_v_numpy',
-
-    'set_oo_data',
-    'get_oo_frame',
-    'get_oo_proper',
-
-    'set_vv_data',
-    'get_vv_frame',
-    'get_vv_proper',
-
-    'set_oa_data',
-    'get_oa_frame',
-    'get_oa_proper',
-
-    'set_va_data',
-    'get_va_frame',
-    'get_va_proper',
-
-    'set_vo_data',
-    'get_vo_frame',
-    'get_vo_proper',
-
-    'has_data',
+    "slice",
+    "copy_adata",
+    "set_name",
+    "get_name",
+    "set_m_data",
+    "get_m_data",
+    "set_o_data",
+    "get_o_series",
+    "get_o_numpy",
+    "get_o_names",
+    "maybe_o_numpy",
+    "set_v_data",
+    "get_v_series",
+    "get_v_numpy",
+    "get_v_names",
+    "maybe_v_numpy",
+    "set_oo_data",
+    "get_oo_frame",
+    "get_oo_proper",
+    "set_vv_data",
+    "get_vv_frame",
+    "get_vv_proper",
+    "set_oa_data",
+    "get_oa_frame",
+    "get_oa_proper",
+    "set_va_data",
+    "get_va_frame",
+    "get_va_proper",
+    "set_vo_data",
+    "get_vo_frame",
+    "get_vo_proper",
+    "has_data",
 ]
 
 
@@ -135,7 +132,7 @@ def slice(  # pylint: disable=redefined-builtin,too-many-branches,too-many-state
     share_derived: bool = True,
     top_level: bool = True,
 ) -> AnnData:
-    '''
+    """
     Return new annotated data which includes a subset of the full ``adata``.
 
     If ``name`` is not specified, the data will be unnamed. Otherwise, if it starts with a ``.``, it
@@ -152,8 +149,8 @@ def slice(  # pylint: disable=redefined-builtin,too-many-branches,too-many-state
     If the slice happens to be the full original data, then this becomes equivalent to
     :py:func:`copy_adata`, and by default this will ``share_derived`` (share the derived data
     cache).
-    '''
-    assert '__x__' not in adata.layers
+    """
+    assert "__x__" not in adata.layers
 
     is_same_obs: Optional[bool] = None
     if obs is None:
@@ -165,7 +162,7 @@ def slice(  # pylint: disable=redefined-builtin,too-many-branches,too-many-state
             is_same_obs = False
 
         obs = utt.to_numpy_vector(obs)
-        if obs.dtype == 'bool':
+        if obs.dtype == "bool":
             assert obs.size == adata.n_obs
             assert np.any(obs)
             is_same_obs = bool(np.all(obs))
@@ -180,59 +177,53 @@ def slice(  # pylint: disable=redefined-builtin,too-many-branches,too-many-state
             is_same_vars = False
 
         vars = utt.to_numpy_vector(vars)
-        if vars.dtype == 'bool':
+        if vars.dtype == "bool":
             assert vars.size == adata.n_vars
             assert np.any(vars)
             is_same_vars = bool(np.all(vars))
 
     if is_same_obs and is_same_vars:
-        bdata = copy_adata(adata, name=name, share_derived=share_derived,
-                           top_level=top_level)
+        bdata = copy_adata(adata, name=name, share_derived=share_derived, top_level=top_level)
 
     else:
         if is_same_vars:
-            replaced = _replace_with_layout(adata, 'row_major')
+            replaced = _replace_with_layout(adata, "row_major")
         elif is_same_obs:
-            replaced = _replace_with_layout(adata, 'column_major')
+            replaced = _replace_with_layout(adata, "column_major")
         else:
             replaced = {}
 
         try:
-            with utm.timed_step('adata.slice'):
+            with utm.timed_step("adata.slice"):
                 bdata = adata[obs, vars].copy()
         finally:
             _replace_back(adata, replaced)
 
         set_name(bdata, name)
 
-        if hasattr(bdata, '__is_top_level__'):
-            delattr(bdata, '__is_top_level__')
-        if hasattr(bdata, '__derived__'):
-            delattr(bdata, '__derived__')
+        if hasattr(bdata, "__is_top_level__"):
+            delattr(bdata, "__is_top_level__")
+        if hasattr(bdata, "__derived__"):
+            delattr(bdata, "__derived__")
 
-        if share_derived \
-                and (is_same_obs or is_same_obs is None) \
-                and (is_same_vars or is_same_vars is None):
+        if share_derived and (is_same_obs or is_same_obs is None) and (is_same_vars or is_same_vars is None):
 
             if is_same_obs is None:
-                is_same_obs = bdata.n_obs == adata.n_obs \
-                    and bool(np.all(bdata.obs_names == adata.obs_names))
+                is_same_obs = bdata.n_obs == adata.n_obs and bool(np.all(bdata.obs_names == adata.obs_names))
 
             if is_same_vars is None:
-                is_same_vars = bdata.n_vars == adata.n_vars \
-                    and bool(np.all(bdata.var_names == adata.var_names))
+                is_same_vars = bdata.n_vars == adata.n_vars and bool(np.all(bdata.var_names == adata.var_names))
 
             if is_same_obs and is_same_vars:
-                if not hasattr(adata, '__derived__'):
-                    setattr(adata, '__derived__', {})
-                setattr(bdata, '__derived__', getattr(adata, '__derived__'))
+                if not hasattr(adata, "__derived__"):
+                    setattr(adata, "__derived__", {})
+                setattr(bdata, "__derived__", getattr(adata, "__derived__"))
 
         if top_level:
             utl.top_level(bdata)
 
     if utl.logging_calc():
-        utl.log_calc(  #
-            f'slice {get_name(adata, "unnamed")} into {get_name(bdata, "unnamed")} shape {bdata.shape}')
+        utl.log_calc(f'slice {get_name(adata, "unnamed")} into {get_name(bdata, "unnamed")} shape {bdata.shape}')  #
 
     if track_obs is not None:
         set_o_data(bdata, track_obs, np.arange(adata.n_obs)[obs])
@@ -247,10 +238,10 @@ def slice(  # pylint: disable=redefined-builtin,too-many-branches,too-many-state
 def _replace_with_layout(adata: AnnData, layout: str) -> Dict[str, utt.Matrix]:
     replaced: Dict[str, utt.Matrix] = {}
 
-    matrix: utt.Matrix = adata.X  # type: ignore
+    matrix: utt.Matrix = adata.X
     if not utt.is_layout(matrix, layout):
-        replaced['__x__'] = matrix
-        adata.X = get_vo_proper(adata, '__x__', layout=layout)
+        replaced["__x__"] = matrix
+        adata.X = get_vo_proper(adata, "__x__", layout=layout)
 
     for name in adata.layers:
         matrix = adata.layers[name]
@@ -264,7 +255,7 @@ def _replace_with_layout(adata: AnnData, layout: str) -> Dict[str, utt.Matrix]:
 @utm.timed_call()
 def _replace_back(adata: AnnData, replaced: Dict[str, utt.Matrix]) -> None:
     for name, matrix in replaced.items():
-        if name == '__x__':
+        if name == "__x__":
             adata.X = matrix
         else:
             adata.layers[name] = matrix
@@ -272,13 +263,9 @@ def _replace_back(adata: AnnData, replaced: Dict[str, utt.Matrix]) -> None:
 
 @utm.timed_call()
 def copy_adata(
-    adata: AnnData,
-    *,
-    name: Optional[str] = None,
-    share_derived: bool = True,
-    top_level: bool = True
+    adata: AnnData, *, name: Optional[str] = None, share_derived: bool = True, top_level: bool = True
 ) -> AnnData:
-    '''
+    """
     Return a copy of some annotated ``adata``.
 
     If ``name`` is not specified, the data will be unnamed. Otherwise, if it starts with a ``.``, it
@@ -293,22 +280,22 @@ def copy_adata(
         In general we assume annotated data is **not** modified in-place, but it might make sense to
         create a copy (**not** sharing derived data), modify it immediately (before accessing data
         in a specific layout), and then proceed to process it without further modifications.
-    '''
-    with utm.timed_step('adata.copy'):
+    """
+    with utm.timed_step("adata.copy"):
         bdata = adata.copy()
 
     set_name(bdata, name)
 
-    if hasattr(bdata, '__is_top_level__'):
-        delattr(bdata, '__is_top_level__')
+    if hasattr(bdata, "__is_top_level__"):
+        delattr(bdata, "__is_top_level__")
 
     if share_derived:
-        if not hasattr(adata, '__derived__'):
-            setattr(adata, '__derived__', {})
-        setattr(bdata, '__derived__', getattr(adata, '__derived__'))
+        if not hasattr(adata, "__derived__"):
+            setattr(adata, "__derived__", {})
+        setattr(bdata, "__derived__", getattr(adata, "__derived__"))
     else:
-        if hasattr(bdata, '__derived__'):
-            delattr(bdata, '__derived__')
+        if hasattr(bdata, "__derived__"):
+            delattr(bdata, "__derived__")
 
     if top_level:
         utl.top_level(bdata)
@@ -320,38 +307,38 @@ def has_data(
     name: str,
     layout: Optional[str] = None,
 ) -> bool:
-    '''
+    """
     Test whether we have the specified data.
 
     If the data is per-variable-per-observation, and ``layout`` is specified (one of ``row_major``
     and ``column_major``), it returns whether the specific data layout is available in the cache,
     without having to re-layout existing data.
-    '''
+    """
     assert layout is None or layout in utt.LAYOUT_OF_AXIS
 
     derived: Union[Dict[str, Any], List[str]] = []
-    if hasattr(adata, '__derived__'):
-        derived = getattr(adata, '__derived__')
+    if hasattr(adata, "__derived__"):
+        derived = getattr(adata, "__derived__")
 
-    if name == '__x__':
+    if name == "__x__":
         if layout is None:
             return True
-        matrix: utt.Matrix = adata.X  # type: ignore
-        return utt.matrix_layout(matrix) == layout \
-            or f'vo:__x__:{layout}' in derived
+        matrix: utt.Matrix = adata.X
+        return utt.matrix_layout(matrix) == layout or f"vo:__x__:{layout}" in derived
 
-    for per, annotations in (('vo', adata.layers),
-                             ('oo', adata.obsp),
-                             ('vv', adata.varp),
-                             ('va', adata.obsm),
-                             ('oa', adata.varm)):
+    for per, annotations in (
+        ("vo", adata.layers),
+        ("oo", adata.obsp),
+        ("vv", adata.varp),
+        ("va", adata.obsm),
+        ("oa", adata.varm),
+    ):
         if name not in annotations:
             continue
         if layout is None:
             return True
         matrix = annotations[name]
-        return utt.matrix_layout(matrix) == layout \
-            or f'{per}:{name}:{layout}' in derived
+        return utt.matrix_layout(matrix) == layout or f"{per}:{name}:{layout}" in derived
 
     assert layout is None
 
@@ -368,13 +355,13 @@ def get_m_data(
     *,
     formatter: Optional[Callable[[Any], Any]] = None,
 ) -> Any:
-    '''
+    """
     Get metadata (unstructured annotation) in ``adata`` by its ``name``.
-    '''
+    """
     data = adata.uns.get(name)
     if data is None:
-        raise _unknown_data(adata, name, 'm')
-    utl.log_get(adata, 'm', name, data, formatter=formatter)
+        raise _unknown_data(adata, name, "m")
+    utl.log_get(adata, "m", name, data, formatter=formatter)
     return data
 
 
@@ -386,9 +373,9 @@ def _get_o_data(
     formatter: Optional[Callable[[Any], Any]] = None,
 ) -> Any:
     if not isinstance(name, str):
-        log_name = '<data>'
+        log_name = "<data>"
     else:
-        if name.endswith('|sum'):
+        if name.endswith("|sum"):
             name = name[:-4]
             assert not sum
             sum = True
@@ -397,12 +384,11 @@ def _get_o_data(
     if sum:
         if formatter is None:
             formatter = utl.sizes_description
-        data = _get_vo_sum_data(adata, 'row', name)
-        log_name += '|sum'
+        data = _get_vo_sum_data(adata, "row", name)
+        log_name += "|sum"
     else:
-        data = _get_shaped_data(adata, 'o', adata.obs,
-                                shape=(adata.n_obs,), name=name)
-    utl.log_get(adata, 'o', log_name, data, formatter=formatter)
+        data = _get_shaped_data(adata, "o", adata.obs, shape=(adata.n_obs,), name=name)
+    utl.log_get(adata, "o", log_name, data, formatter=formatter)
     return data
 
 
@@ -413,12 +399,12 @@ def get_o_series(
     sum: bool = False,  # pylint: disable=redefined-builtin
     formatter: Optional[Callable[[Any], Any]] = None,
 ) -> utt.PandasSeries:
-    '''
+    """
     Get per-observation (cell) data in ``adata`` by its ``name`` as a pandas series.
 
     If ``name`` is a string, it is the name of a per-observation annotation to fetch.
     Otherwise, it should be some vector of data of the appropriate size.
-    '''
+    """
     data = _get_o_data(adata, name, sum=sum, formatter=formatter)
     series = utt.maybe_pandas_series(data)
     if series is None:
@@ -433,7 +419,7 @@ def get_o_numpy(
     sum: bool = False,  # pylint: disable=redefined-builtin
     formatter: Optional[Callable[[Any], Any]] = None,
 ) -> utt.NumpyVector:
-    '''
+    """
     Get per-observation (cell) data in ``adata`` by its ``name`` as a Numpy array.
 
     If ``name`` is a string, it is the name of a per-observation annotation to fetch.
@@ -441,7 +427,7 @@ def get_o_numpy(
 
     If ``sum`` is ``True``, then ``name`` should be the name of a per-observation-per-variable
     annotation, or a matrix, and this will return the sum (per row) of this data.
-    '''
+    """
     data = _get_o_data(adata, name, sum=sum, formatter=formatter)
     return utt.to_numpy_vector(data)
 
@@ -449,11 +435,11 @@ def get_o_numpy(
 def get_o_names(
     adata: AnnData,
 ) -> utt.NumpyVector:
-    '''
+    """
     Get a numpy vector of observation names.
-    '''
+    """
     data = utt.to_numpy_vector(adata.obs_names)
-    utl.log_get(adata, 'o', '__name__', data)
+    utl.log_get(adata, "o", "__name__", data)
     return data
 
 
@@ -464,9 +450,9 @@ def maybe_o_numpy(
     sum: bool = False,  # pylint: disable=redefined-builtin
     formatter: Optional[Callable[[Any], Any]] = None,
 ) -> Optional[utt.NumpyVector]:
-    '''
+    """
     Similar to :py:func:`get_o_numpy`, but if ``name`` is ``None``, return ``None``.
-    '''
+    """
     if name is None:
         return None
     return get_o_numpy(adata, name, sum=sum, formatter=formatter)
@@ -480,9 +466,9 @@ def _get_v_data(
     formatter: Optional[Callable[[Any], Any]] = None,
 ) -> Any:
     if not isinstance(name, str):
-        log_name = '<data>'
+        log_name = "<data>"
     else:
-        if isinstance(name, str) and name.endswith('|sum'):
+        if isinstance(name, str) and name.endswith("|sum"):
             name = name[:-4]
             assert not sum
             sum = True
@@ -491,12 +477,11 @@ def _get_v_data(
     if sum:
         if formatter is None:
             formatter = utl.sizes_description
-        data = _get_vo_sum_data(adata, 'column', name)
-        log_name += '|sum'
+        data = _get_vo_sum_data(adata, "column", name)
+        log_name += "|sum"
     else:
-        data = _get_shaped_data(adata, 'v', adata.var,
-                                shape=(adata.n_vars,), name=name)
-    utl.log_get(adata, 'v', log_name, data, formatter=formatter)
+        data = _get_shaped_data(adata, "v", adata.var, shape=(adata.n_vars,), name=name)
+    utl.log_get(adata, "v", log_name, data, formatter=formatter)
     return data
 
 
@@ -507,12 +492,12 @@ def get_v_series(
     sum: bool = False,  # pylint: disable=redefined-builtin
     formatter: Optional[Callable[[Any], Any]] = None,
 ) -> utt.PandasSeries:
-    '''
+    """
     Get per-variable (gene) data in ``adata`` by its ``name`` as a pandas series.
 
     If ``name`` is a string, it is the name of a per-variable annotation to fetch.
     Otherwise, it should be some vector of data of the appropriate size.
-    '''
+    """
     data = _get_v_data(adata, name, sum=sum, formatter=formatter)
     series = utt.maybe_pandas_series(data)
     if series is None:
@@ -527,12 +512,12 @@ def get_v_numpy(
     sum: bool = False,  # pylint: disable=redefined-builtin
     formatter: Optional[Callable[[Any], Any]] = None,
 ) -> utt.NumpyVector:
-    '''
+    """
     Get per-variable (gene) data in ``adata`` by its ``name`` as a numpy array.
 
     If ``name`` is a string, it is the name of a per-variable annotation to fetch.
     Otherwise, it should be some vector of data of the appropriate size.
-    '''
+    """
     data = _get_v_data(adata, name, sum=sum, formatter=formatter)
     return utt.to_numpy_vector(data)
 
@@ -540,11 +525,11 @@ def get_v_numpy(
 def get_v_names(
     adata: AnnData,
 ) -> utt.NumpyVector:
-    '''
+    """
     Get a numpy vector of variable names.
-    '''
+    """
     data = utt.to_numpy_vector(adata.var_names)
-    utl.log_get(adata, 'v', '__name__', data)
+    utl.log_get(adata, "v", "__name__", data)
     return data
 
 
@@ -555,9 +540,9 @@ def maybe_v_numpy(
     sum: bool = False,  # pylint: disable=redefined-builtin
     formatter: Optional[Callable[[Any], Any]] = None,
 ) -> Optional[utt.NumpyVector]:
-    '''
+    """
     Similar to :py:func:`get_v_numpy`, but if ``name`` is ``None``, return ``None``.
-    '''
+    """
     if name is None:
         return None
     return get_v_numpy(adata, name, sum=sum, formatter=formatter)
@@ -570,10 +555,8 @@ def _get_oo_data(
     layout: Optional[str] = None,
     formatter: Optional[Callable[[Any], Any]] = None,
 ) -> utt.Matrix:
-    data = _get_layout_data(adata, 'oo', adata.obsp,
-                            shape=(adata.n_obs, adata.n_obs),
-                            name=name, layout=layout)
-    utl.log_get(adata, 'oo', name, data, formatter=formatter)
+    data = _get_layout_data(adata, "oo", adata.obsp, shape=(adata.n_obs, adata.n_obs), name=name, layout=layout)
+    utl.log_get(adata, "oo", name, data, formatter=formatter)
     return data
 
 
@@ -584,7 +567,7 @@ def get_oo_frame(
     layout: Optional[str] = None,
     formatter: Optional[Callable[[Any], Any]] = None,
 ) -> utt.PandasFrame:
-    '''
+    """
     Get per-observation-per-observation (per-cell-per-cell) data as a pandas data frame.
 
     If ``name`` is a string, it is the name of a per-variable annotation to fetch.
@@ -593,13 +576,11 @@ def get_oo_frame(
     If ``layout`` (default: {layout}) is specified, it must be one of ``row_major`` or
     ``column_major``. If this requires relayout of the data, the result is cached in a hidden data
     member for future reuse.
-    '''
+    """
     data = _get_oo_data(adata, name, layout=layout, formatter=formatter)
     frame = utt.maybe_pandas_frame(data)
     if frame is None:
-        frame = utt.to_pandas_frame(utt.to_proper_matrix(data),
-                                    index=adata.obs_names,
-                                    columns=adata.obs_names)
+        frame = utt.to_pandas_frame(utt.to_proper_matrix(data), index=adata.obs_names, columns=adata.obs_names)
     return frame
 
 
@@ -610,11 +591,11 @@ def get_oo_proper(
     layout: Optional[str] = None,
     formatter: Optional[Callable[[Any], Any]] = None,
 ) -> utt.ProperMatrix:
-    '''
+    """
     Same as ``get_oo_data`` but returns a :py:const:`metacells.utilities.typing.ProperMatrix`.
-    '''
+    """
     data = _get_oo_data(adata, name, layout=layout, formatter=formatter)
-    return utt.to_proper_matrix(data, default_layout=layout or 'row_major')
+    return utt.to_proper_matrix(data, default_layout=layout or "row_major")
 
 
 def _get_vv_data(
@@ -624,10 +605,8 @@ def _get_vv_data(
     layout: Optional[str] = None,
     formatter: Optional[Callable[[Any], Any]] = None,
 ) -> utt.Matrix:
-    data = _get_layout_data(adata, 'vv', adata.varp,
-                            shape=(adata.n_vars, adata.n_vars),
-                            name=name, layout=layout)
-    utl.log_get(adata, 'vv', name, data, formatter=formatter)
+    data = _get_layout_data(adata, "vv", adata.varp, shape=(adata.n_vars, adata.n_vars), name=name, layout=layout)
+    utl.log_get(adata, "vv", name, data, formatter=formatter)
     return data
 
 
@@ -638,7 +617,7 @@ def get_vv_frame(
     layout: Optional[str] = None,
     formatter: Optional[Callable[[Any], Any]] = None,
 ) -> utt.PandasFrame:
-    '''
+    """
     Get per-variable-per-variable (per-gene-per-gene) data as a pandas data frame.
 
     If ``name`` is a string, it is the name of a per-variable annotation to fetch.
@@ -647,13 +626,11 @@ def get_vv_frame(
     If ``layout`` (default: {layout}) is specified, it must be one of ``row_major`` or
     ``column_major``. If this requires relayout of the data, the result is cached in a hidden data
     member for future reuse.
-    '''
+    """
     data = _get_vv_data(adata, name, layout=layout, formatter=formatter)
     frame = utt.maybe_pandas_frame(data)
     if frame is None:
-        frame = utt.to_pandas_frame(utt.to_proper_matrix(data),
-                                    index=adata.var_names,
-                                    columns=adata.var_names)
+        frame = utt.to_pandas_frame(utt.to_proper_matrix(data), index=adata.var_names, columns=adata.var_names)
     return frame
 
 
@@ -664,11 +641,11 @@ def get_vv_proper(
     layout: Optional[str] = None,
     formatter: Optional[Callable[[Any], Any]] = None,
 ) -> utt.ProperMatrix:
-    '''
+    """
     Same as ``get_vv_data`` but returns a :py:const:`metacells.utilities.typing.ProperMatrix`.
-    '''
+    """
     data = _get_vv_data(adata, name, layout=layout, formatter=formatter)
-    return utt.to_proper_matrix(data, default_layout=layout or 'row_major')
+    return utt.to_proper_matrix(data, default_layout=layout or "row_major")
 
 
 def _get_oa_data(
@@ -678,10 +655,8 @@ def _get_oa_data(
     layout: Optional[str] = None,
     formatter: Optional[Callable[[Any], Any]] = None,
 ) -> utt.Matrix:
-    data = _get_layout_data(adata, 'oa', adata.obsm,
-                            shape=(adata.n_obs, 0),
-                            name=name, layout=layout)
-    utl.log_get(adata, 'oa', name, data, formatter=formatter)
+    data = _get_layout_data(adata, "oa", adata.obsm, shape=(adata.n_obs, 0), name=name, layout=layout)
+    utl.log_get(adata, "oa", name, data, formatter=formatter)
     return data
 
 
@@ -693,7 +668,7 @@ def get_oa_frame(
     layout: Optional[str] = None,
     formatter: Optional[Callable[[Any], Any]] = None,
 ) -> utt.PandasFrame:
-    '''
+    """
     Get per-observation-per-any (per-cell-per-any) data as a pandas data frame.
 
     Rows are observations (cells), indexed by the observation names (typically cell barcode).
@@ -705,13 +680,11 @@ def get_oa_frame(
     If ``layout`` (default: {layout}) is specified, it must be one of ``row_major`` or
     ``column_major``. If this requires relayout of the data, the result is cached in a hidden data
     member for future reuse.
-    '''
+    """
     data = _get_oa_data(adata, name, layout=layout, formatter=formatter)
     frame = utt.maybe_pandas_frame(data)
     if frame is None:
-        frame = utt.to_pandas_frame(utt.to_proper_matrix(data),
-                                    index=adata.obs_names,
-                                    columns=columns)
+        frame = utt.to_pandas_frame(utt.to_proper_matrix(data), index=adata.obs_names, columns=columns)
     return frame
 
 
@@ -722,11 +695,11 @@ def get_oa_proper(
     layout: Optional[str] = None,
     formatter: Optional[Callable[[Any], Any]] = None,
 ) -> utt.ProperMatrix:
-    '''
+    """
     Same as ``get_oa_data`` but returns a :py:const:`metacells.utilities.typing.ProperMatrix`.
-    '''
+    """
     data = _get_oa_data(adata, name, layout=layout, formatter=formatter)
-    return utt.to_proper_matrix(data, default_layout=layout or 'row_major')
+    return utt.to_proper_matrix(data, default_layout=layout or "row_major")
 
 
 def _get_va_data(
@@ -736,10 +709,8 @@ def _get_va_data(
     layout: Optional[str] = None,
     formatter: Optional[Callable[[Any], Any]] = None,
 ) -> utt.Matrix:
-    data = _get_layout_data(adata, 'va', adata.varm,
-                            shape=(adata.n_vars, 0),
-                            name=name, layout=layout)
-    utl.log_get(adata, 'va', name, data, formatter=formatter)
+    data = _get_layout_data(adata, "va", adata.varm, shape=(adata.n_vars, 0), name=name, layout=layout)
+    utl.log_get(adata, "va", name, data, formatter=formatter)
     return data
 
 
@@ -751,7 +722,7 @@ def get_va_frame(
     layout: Optional[str] = None,
     formatter: Optional[Callable[[Any], Any]] = None,
 ) -> utt.PandasFrame:
-    '''
+    """
     Get per-variable-per-any (per-cell-per-any) data as a pandas data frame.
 
     Rows are variables (genes), indexed by their names.
@@ -763,13 +734,11 @@ def get_va_frame(
     If ``layout`` (default: {layout}) is specified, it must be one of ``row_major`` or
     ``column_major``. If this requires relayout of the data, the result is cached in a hidden data
     member for future reuse.
-    '''
+    """
     data = _get_va_data(adata, name, layout=layout, formatter=formatter)
     frame = utt.maybe_pandas_frame(data)
     if frame is None:
-        frame = utt.to_pandas_frame(utt.to_proper_matrix(data),
-                                    index=adata.var_names,
-                                    columns=columns)
+        frame = utt.to_pandas_frame(utt.to_proper_matrix(data), index=adata.var_names, columns=columns)
     return frame
 
 
@@ -780,11 +749,11 @@ def get_va_proper(
     layout: Optional[str] = None,
     formatter: Optional[Callable[[Any], Any]] = None,
 ) -> utt.ProperMatrix:
-    '''
+    """
     Same as ``get_va_data`` but returns a :py:const:`metacells.utilities.typing.ProperMatrix`.
-    '''
+    """
     data = _get_va_data(adata, name, layout=layout, formatter=formatter)
-    return utt.to_proper_matrix(data, default_layout=layout or 'row_major')
+    return utt.to_proper_matrix(data, default_layout=layout or "row_major")
 
 
 def _get_vo_data(
@@ -795,23 +764,21 @@ def _get_vo_data(
     formatter: Optional[Callable[[Any], Any]] = None,
 ) -> utt.Matrix:
     if name is None:
-        name = '__x__'
+        name = "__x__"
 
-    data = _get_layout_data(adata, 'vo', adata.layers,
-                            shape=(adata.n_obs, adata.n_vars),
-                            name=name, layout=layout)
-    utl.log_get(adata, 'vo', name, data, formatter=formatter)
+    data = _get_layout_data(adata, "vo", adata.layers, shape=(adata.n_obs, adata.n_vars), name=name, layout=layout)
+    utl.log_get(adata, "vo", name, data, formatter=formatter)
     return data
 
 
 def get_vo_frame(
     adata: AnnData,
-    name: Union[str, utt.Matrix] = '__x__',
+    name: Union[str, utt.Matrix] = "__x__",
     *,
     layout: Optional[str] = None,
     formatter: Optional[Callable[[Any], Any]] = None,
 ) -> utt.PandasFrame:
-    '''
+    """
     Get per-variable-per-observation (per-gene-per-cell) data as a pandas data frame.
 
     Rows are observations (cells), indexed by the observation names (typically cell barcode).
@@ -823,28 +790,26 @@ def get_vo_frame(
     If ``layout`` (default: {layout}) is specified, it must be one of ``row_major`` or
     ``column_major``. If this requires relayout of the data, the result is cached in a hidden data
     member for future reuse.
-    '''
+    """
     data = _get_vo_data(adata, name, layout=layout, formatter=formatter)
     frame = utt.maybe_pandas_frame(data)
     if frame is None:
-        frame = utt.to_pandas_frame(utt.to_proper_matrix(data),
-                                    index=adata.obs_names,
-                                    columns=adata.var_names)
+        frame = utt.to_pandas_frame(utt.to_proper_matrix(data), index=adata.obs_names, columns=adata.var_names)
     return frame
 
 
 def get_vo_proper(
     adata: AnnData,
-    name: Union[str, utt.Matrix] = '__x__',
+    name: Union[str, utt.Matrix] = "__x__",
     *,
     layout: Optional[str] = None,
     formatter: Optional[Callable[[Any], Any]] = None,
 ) -> utt.ProperMatrix:
-    '''
+    """
     Same as ``get_vo_data`` but returns a :py:const:`metacells.utilities.typing.ProperMatrix`.
-    '''
+    """
     data = _get_vo_data(adata, name, layout=layout, formatter=formatter)
-    return utt.to_proper_matrix(data, default_layout=layout or 'row_major')
+    return utt.to_proper_matrix(data, default_layout=layout or "row_major")
 
 
 def _get_vo_sum_data(
@@ -857,18 +822,18 @@ def _get_vo_sum_data(
         assert name.shape == adata.shape  # type: ignore
         return utc.sum_per(name, per=per)  # type: ignore
 
-    if not hasattr(adata, '__derived__'):
-        derived: Dict[str, utt.NumpyVector] = dict()
-        setattr(adata, '__derived__', derived)
+    if not hasattr(adata, "__derived__"):
+        derived: Dict[str, utt.NumpyVector] = {}
+        setattr(adata, "__derived__", derived)
     else:
-        derived = getattr(adata, '__derived__')
+        derived = getattr(adata, "__derived__")
 
-    sum_name = f'vo:{name}:sum_per_{per}'
+    sum_name = f"vo:{name}:sum_per_{per}"
     sum_data = derived.get(sum_name)
     if sum_data is not None:
         return sum_data
 
-    data = get_vo_proper(adata, name, layout=f'{per}_major')
+    data = get_vo_proper(adata, name, layout=f"{per}_major")
     sum_data = utc.sum_per(data, per=per)
     derived[sum_name] = sum_data
     return sum_data
@@ -894,13 +859,13 @@ def _get_layout_data(
         return data
 
     assert layout in utt.LAYOUT_OF_AXIS
-    layout_name = '%s:%s:%s' % (per, name, layout)
+    layout_name = f"{per}:{name}:{layout}"
 
-    if not hasattr(adata, '__derived__'):
-        derived: Dict[str, utt.ProperMatrix] = dict()
-        setattr(adata, '__derived__', derived)
+    if not hasattr(adata, "__derived__"):
+        derived: Dict[str, utt.ProperMatrix] = {}
+        setattr(adata, "__derived__", derived)
     else:
-        derived = getattr(adata, '__derived__')
+        derived = getattr(adata, "__derived__")
 
     layout_data = derived.get(layout_name)
     if layout_data is not None:
@@ -925,7 +890,7 @@ def _get_shaped_data(
     name: Union[str, utt.Shaped],
 ) -> Any:
     if isinstance(name, str):
-        if per == 'vo' and name == '__x__':
+        if per == "vo" and name == "__x__":
             data = _fix_data(adata.X)
         else:
             if name not in annotations:  # type: ignore
@@ -951,55 +916,53 @@ def _fix_data(data: Any) -> Any:
     return data
 
 
-def _annotation_per(  # pylint: disable=too-many-return-statements
-    adata: AnnData, per: str
-) -> Annotations:
-    if per == 'm':
+def _annotation_per(adata: AnnData, per: str) -> Annotations:  # pylint: disable=too-many-return-statements
+    if per == "m":
         return adata.uns
-    if per == 'vo':
+    if per == "vo":
         return adata.layers
-    if per == 'v':
+    if per == "v":
         return adata.var
-    if per == 'o':
+    if per == "o":
         return adata.obs
-    if per == 'vv':
+    if per == "vv":
         return adata.varp
-    if per == 'oo':
+    if per == "oo":
         return adata.obsp
-    if per == 'va':
+    if per == "va":
         return adata.varm
-    if per == 'oa':
+    if per == "oa":
         return adata.obsm
-    raise ValueError('unknown per: %s' % per)
+    raise ValueError(f"unknown per: {per}")
 
 
 def set_name(adata: AnnData, name: Optional[str]) -> None:
-    '''
+    """
     Set the ``name`` of the data (for log messages).
 
     If the name starts with ``.`` it is appended to the current name, if any.
-    '''
+    """
     if name is None:
-        if '__name__' in adata.uns:
-            del adata.uns['__name__']
+        if "__name__" in adata.uns:
+            del adata.uns["__name__"]
         return
 
-    if name[0] == '.':
+    if name[0] == ".":
         old_name = get_name(adata)
         if old_name is None:
             name = name[1:]
         else:
             name = old_name + name
-    adata.uns['__name__'] = name
+    adata.uns["__name__"] = name
 
 
 def get_name(adata: AnnData, default: Optional[str] = None) -> Optional[str]:
-    '''
+    """
     Return the name of the data (for log messages), if any.
 
     If no name was set, returns the ``default``.
-    '''
-    return adata.uns.get('__name__', default)
+    """
+    return adata.uns.get("__name__", default)
 
 
 def set_m_data(
@@ -1009,12 +972,12 @@ def set_m_data(
     *,
     formatter: Optional[Callable[[Any], Any]] = None,
 ) -> Any:
-    '''
+    """
     Set unstructured data.
 
     If ``formatter`` is specified, its results is used when logging the operation.
-    '''
-    utl.log_set(adata, 'm', name, data, formatter=formatter)
+    """
+    utl.log_set(adata, "m", name, data, formatter=formatter)
     adata.uns[name] = data
 
 
@@ -1025,12 +988,12 @@ def set_o_data(
     *,
     formatter: Optional[Callable[[Any], Any]] = None,
 ) -> Any:
-    '''
+    """
     Set per-observation (cell) data.
 
     If ``formatter`` is specified, its results is used when logging the operation.
-    '''
-    utl.log_set(adata, 'o', name, data, formatter=formatter)
+    """
+    utl.log_set(adata, "o", name, data, formatter=formatter)
 
     if not isinstance(data, list):
         utt.mustbe_canonical(data)
@@ -1041,18 +1004,14 @@ def set_o_data(
 
 
 def set_v_data(
-    adata: AnnData,
-    name: str,
-    data: utt.NumpyVector,
-    *,
-    formatter: Optional[Callable[[Any], Any]] = None
+    adata: AnnData, name: str, data: utt.NumpyVector, *, formatter: Optional[Callable[[Any], Any]] = None
 ) -> Any:
-    '''
+    """
     Set per-variable (gene) data.
 
     If ``formatter`` is specified, its results is used when logging the operation.
-    '''
-    utl.log_set(adata, 'v', name, data, formatter=formatter)
+    """
+    utl.log_set(adata, "v", name, data, formatter=formatter)
 
     utt.mustbe_canonical(data)
     if not utt.frozen(data):
@@ -1062,18 +1021,14 @@ def set_v_data(
 
 
 def set_oo_data(
-    adata: AnnData,
-    name: str,
-    data: utt.ProperMatrix,
-    *,
-    formatter: Optional[Callable[[Any], Any]] = None
+    adata: AnnData, name: str, data: utt.ProperMatrix, *, formatter: Optional[Callable[[Any], Any]] = None
 ) -> Any:
-    '''
+    """
     Set per-observation-per-observation (cell) data.
 
     If ``formatter`` is specified, its results is used when logging the operation.
-    '''
-    utl.log_set(adata, 'oo', name, data, formatter=formatter)
+    """
+    utl.log_set(adata, "oo", name, data, formatter=formatter)
 
     utt.mustbe_canonical(data)
     if not utt.frozen(data):
@@ -1083,18 +1038,14 @@ def set_oo_data(
 
 
 def set_vv_data(
-    adata: AnnData,
-    name: str,
-    data: utt.ProperMatrix,
-    *,
-    formatter: Optional[Callable[[Any], Any]] = None
+    adata: AnnData, name: str, data: utt.ProperMatrix, *, formatter: Optional[Callable[[Any], Any]] = None
 ) -> Any:
-    '''
+    """
     Set per-variable-per-variable (gene) data.
 
     If ``formatter`` is specified, its results is used when logging the operation.
-    '''
-    utl.log_set(adata, 'vv', name, data, formatter=formatter)
+    """
+    utl.log_set(adata, "vv", name, data, formatter=formatter)
 
     utt.mustbe_canonical(data)
     if not utt.frozen(data):
@@ -1104,18 +1055,14 @@ def set_vv_data(
 
 
 def set_oa_data(
-    adata: AnnData,
-    name: str,
-    data: utt.ProperMatrix,
-    *,
-    formatter: Optional[Callable[[Any], Any]] = None
+    adata: AnnData, name: str, data: utt.ProperMatrix, *, formatter: Optional[Callable[[Any], Any]] = None
 ) -> Any:
-    '''
+    """
     Set per-observation-per-any (cell) data.
 
     If ``formatter`` is specified, its results is used when logging the operation.
-    '''
-    utl.log_set(adata, 'oa', name, data, formatter=formatter)
+    """
+    utl.log_set(adata, "oa", name, data, formatter=formatter)
 
     utt.mustbe_canonical(data)
     if not utt.frozen(data):
@@ -1125,18 +1072,14 @@ def set_oa_data(
 
 
 def set_va_data(
-    adata: AnnData,
-    name: str,
-    data: utt.ProperMatrix,
-    *,
-    formatter: Optional[Callable[[Any], Any]] = None
+    adata: AnnData, name: str, data: utt.ProperMatrix, *, formatter: Optional[Callable[[Any], Any]] = None
 ) -> Any:
-    '''
+    """
     Set per-variable-per-any (gene) data.
 
     If ``formatter`` is specified, its results is used when logging the operation.
-    '''
-    utl.log_set(adata, 'va', name, data, formatter=formatter)
+    """
+    utl.log_set(adata, "va", name, data, formatter=formatter)
 
     utt.mustbe_canonical(data)
     if not utt.frozen(data):
@@ -1152,35 +1095,35 @@ def set_vo_data(
     *,
     formatter: Optional[Callable[[Any], Any]] = None,
 ) -> Any:
-    '''
+    """
     Set per-variable-per-observation (per-gene-per-cell) data.
-    '''
-    utl.log_set(adata, 'vo', name, data, formatter=formatter)
+    """
+    utl.log_set(adata, "vo", name, data, formatter=formatter)
 
     utt.mustbe_canonical(data)
     if not utt.frozen(data):
         utt.freeze(data)
 
-    if name == '__x__':
+    if name == "__x__":
         adata.X = data
     else:
         adata.layers[name] = data
 
 
 def _unknown_data(adata: AnnData, name: str, per: Optional[str] = None) -> KeyError:
-    texts = ['unknown']
+    texts = ["unknown"]
 
     if per is not None:
-        texts.append(' ')
+        texts.append(" ")
         texts.append(per)
 
-    texts.append(' data')
+    texts.append(" data")
 
     data_name = get_name(adata)
     if data_name is not None:
-        texts.append(': ')
+        texts.append(": ")
         texts.append(data_name)
 
-    texts.append(' name: ')
+    texts.append(" name: ")
     texts.append(name)
-    return KeyError(''.join(texts))
+    return KeyError("".join(texts))
