@@ -1,4 +1,4 @@
-'''
+"""
 Logging
 -------
 
@@ -40,89 +40,106 @@ To improve the log messages, we allow each ``AnnData`` object to have an optiona
 created, its name is extended by some descriptive suffix, so we get names like
 ``full.clean.feature`` to describe the feature data extracted out of the clean data extracted out of
 the full data.
-'''
+"""
 
+import importlib
 import logging
 import sys
 from contextlib import contextmanager
 from datetime import datetime
 from functools import wraps
-from inspect import Parameter, signature
-from logging import (DEBUG, INFO, Formatter, Logger, LogRecord, StreamHandler,
-                     getLogger, setLoggerClass)
+from inspect import Parameter
+from inspect import signature
+from logging import DEBUG
+from logging import INFO
+from logging import Formatter
+from logging import Logger
+from logging import LogRecord
+from logging import StreamHandler
+from logging import getLogger
+from logging import setLoggerClass
 from multiprocessing import Lock
 from threading import current_thread
-from typing import (IO, Any, Callable, Dict, Iterator, List, Optional, Tuple,
-                    TypeVar, Union)
+from typing import IO
+from typing import Any
+from typing import Callable
+from typing import Dict
+from typing import Iterator
+from typing import List
+from typing import Optional
+from typing import Tuple
+from typing import TypeVar
+from typing import Union
 
 import numpy as np
 import pandas as pd  # type: ignore
-from anndata import AnnData
+from anndata import AnnData  # type: ignore
 
 import metacells.utilities.documentation as utd
-import metacells.utilities.parallel as utp
-import metacells.utilities.typing as utt
+
+utp: Any = importlib.import_module("metacells.utilities.parallel")
+utt: Any = importlib.import_module("metacells.utilities.typing")
 
 __all__ = [
-    'setup_logger',
-    'logger',
-    'CALC',
-    'STEP',
-    'PARAM',
-    'logged',
-    'top_level',
-    'log_return',
-    'logging_calc',
-    'log_calc',
-    'log_step',
-    'incremental',
-    'done_incrementals',
-    'cancel_incrementals',
-    'log_set',
-    'log_get',
-    'sizes_description',
-    'fractions_description',
-    'groups_description',
-    'mask_description',
-    'ratio_description',
-    'progress_description',
-    'fraction_description',
-    'fold_description',
+    "setup_logger",
+    "logger",
+    "CALC",
+    "STEP",
+    "PARAM",
+    "logged",
+    "top_level",
+    "log_return",
+    "logging_calc",
+    "log_calc",
+    "log_step",
+    "incremental",
+    "done_incrementals",
+    "cancel_incrementals",
+    "log_set",
+    "log_get",
+    "sizes_description",
+    "fractions_description",
+    "groups_description",
+    "mask_description",
+    "ratio_description",
+    "progress_description",
+    "fraction_description",
+    "fold_description",
 ]
 
 
 class SynchronizedLogger(Logger):
-    '''
+    """
     A logger that synchronizes between the sub-processes.
 
     This ensures logging does not get garbled when when using multi-processing
     (e.g., using :py:func:`metacells.utilities.parallel.parallel_map`).
-    '''
+    """
 
     LOCK = Lock()
 
-    def _log(self, *args: Any, **kwargs: Any) -> Any:  # pylint: disable=signature-differs
+    def _log(self, *args: Any, **kwargs: Any) -> Any:
         with SynchronizedLogger.LOCK:
             super()._log(*args, **kwargs)
 
 
 class LoggingFormatter(Formatter):
-    '''
+    """
     A formatter that uses a decimal point for milliseconds.
-    '''
+    """
 
     def formatTime(self, record: Any, datefmt: Optional[str] = None) -> str:
-        '''
+        """
         Format the time.
-        '''
+        """
         record_datetime = datetime.fromtimestamp(record.created)
         if datefmt is not None:
             assert False
             return record_datetime.strftime(datefmt)
 
-        seconds = record_datetime.strftime('%Y-%m-%d %H:%M:%S')
+        seconds = record_datetime.strftime("%Y-%m-%d %H:%M:%S")
         msecs = round(record.msecs)
-        return f'{seconds}.{msecs:03d}'
+        return f"{seconds}.{msecs:03d}"
 
 
 #: The log level for tracing processing steps.
@@ -134,27 +151,39 @@ PARAM = (2 * DEBUG + 2 * INFO) // 4
 #: The log level for tracing intermediate calculations.
 CALC = (3 * DEBUG + 1 * INFO) // 4
 
-logging.addLevelName(STEP, 'STEP')
-logging.addLevelName(PARAM, 'PARAM')
-logging.addLevelName(CALC, 'CALC')
+logging.addLevelName(STEP, "STEP")
+logging.addLevelName(PARAM, "PARAM")
+logging.addLevelName(CALC, "CALC")
 
 
 class ShortLoggingFormatter(LoggingFormatter):
-    '''
+    """
     Provide short level names.
-    '''
+    """
 
     #: Map the long level names to the fixed-width short level names.
-    SHORT_LEVEL_NAMES = dict(CRITICAL='CRT', CRT='CRT',
-                             ERROR='ERR', ERR='ERR',
-                             WARNING='WRN', WRN='WRN',
-                             INFO='INF', INF='INF',
-                             STEP='STP', STP='STP',
-                             PARAM='PRM', PRM='PRM',
-                             HINT='PRM', HNT='PRM',
-                             CALC='CLC', CLC='CLC',
-                             DEBUG='DBG', DBG='DBG',
-                             NOTSET='NOT', NOT='NOT')
+    SHORT_LEVEL_NAMES = dict(
+        CRITICAL="CRT",
+        CRT="CRT",
+        ERROR="ERR",
+        ERR="ERR",
+        WARNING="WRN",
+        WRN="WRN",
+        INFO="INF",
+        INF="INF",
+        STEP="STP",
+        STP="STP",
+        PARAM="PRM",
+        PRM="PRM",
+        HINT="PRM",
+        HNT="PRM",
+        CALC="CLC",
+        CLC="CLC",
+        DEBUG="DBG",
+        DBG="DBG",
+        NOTSET="NOT",
+        NOT="NOT",
+    )
 
     def format(self, record: LogRecord) -> Any:
         record.levelname = self.SHORT_LEVEL_NAMES[record.levelname]
@@ -175,7 +204,7 @@ def setup_logger(
     name: Optional[str] = None,
     long_level_names: Optional[bool] = None,
 ) -> Logger:
-    '''
+    """
     Setup the global :py:func:`logger`.
 
     .. note::
@@ -208,67 +237,67 @@ def setup_logger(
     If is ``False``, the log level names are shortened to three characters, for consistent
     formatting of indented (nested) log messages. If it is ``None``, no level names are logged at
     all.
-    '''
+    """
     assert utp.is_main_process()
 
     global LOG
     assert LOG is None
 
     if long_level_names is not None:
-        log_format = '%(levelname)s - %(message)s'
+        log_format = "%(levelname)s - %(message)s"
     else:
-        log_format = '%(message)s'
+        log_format = "%(message)s"
 
     if process is None:
         process = level < logging.INFO and utp.get_processors_count() > 1
 
     if process:
-        log_format = '%(threadName)s - ' + log_format
-        current_thread().name = '#0'
+        log_format = "%(threadName)s - " + log_format
+        current_thread().name = "#0"
 
     if name is not None:
-        log_format = name + ' - ' + log_format
+        log_format = name + " - " + log_format
     if time:
-        log_format = '%(asctime)s - ' + log_format
+        log_format = "%(asctime)s - " + log_format
 
     handler = StreamHandler(to)
-    if long_level_names == False:  # pylint: disable=singleton-comparison
+    if long_level_names is False:
         handler.setFormatter(ShortLoggingFormatter(log_format))
     else:
         handler.setFormatter(LoggingFormatter(log_format))
     setLoggerClass(SynchronizedLogger)
-    LOG = getLogger('metacells')
+    LOG = getLogger("metacells")
     LOG.addHandler(handler)
     LOG.setLevel(level)
 
-    LOG.debug('PROCESSORS: %s', utp.get_processors_count())
+    LOG.debug("PROCESSORS: %s", utp.get_processors_count())
     return LOG
 
 
 def logger() -> Logger:
-    '''
+    """
     Access the global logger.
 
     If :py:func:`setup_logger` has not been called yet, this will call it using the default flags.
     You should therefore call :py:func:`setup_logger` as early as possible to ensure you don't end
     up with a misconfigured logger.
-    '''
+    """
     global LOG
     if LOG is None:
         LOG = setup_logger()
     return LOG
 
 
-CALLABLE = TypeVar('CALLABLE')
+CALLABLE = TypeVar("CALLABLE")
 
 CALL_LEVEL = 0
 INDENT_LEVEL = 0
-INDENT_SPACES = '  ' * 1000
+INDENT_SPACES = "  " * 1000
 IS_TOP_LEVEL = True
 
 
 def logged(**kwargs: Callable[[Any], Any]) -> Callable[[CALLABLE], CALLABLE]:
-    '''
+    """
     Automatically wrap each invocation of the decorated function with logging it. Top-level calls
     are logged using the :py:const:`STEP` log level, with parameters logged at the :py:const:`PARAM`
     log level. Nested calls are logged at the ``DEBUG`` log level.
@@ -284,23 +313,24 @@ def logged(**kwargs: Callable[[Any], Any]) -> Callable[[CALLABLE], CALLABLE]:
         @ut.logged()
         def some_function(...):
             ...
-    '''
+    """
     formatter_by_name = kwargs
 
     def wrap(function: Callable) -> Callable:
         parameters = signature(function).parameters
         for name in formatter_by_name:
             if name not in parameters.keys():
-                raise RuntimeError(f'formatter specified for the unknown parameter: {name} '
-                                   f'for the function: {function.__module__}.{function.__qualname__}')
+                raise RuntimeError(
+                    f"formatter specified for the unknown parameter: {name} "
+                    f"for the function: {function.__module__}.{function.__qualname__}"
+                )
         ordered_parameters = list(parameters.values())
 
         @wraps(function)
         def wrapper(*args: Any, **kwargs: Any) -> Any:  # pylint: disable=too-many-branches
             global CALL_LEVEL
             global INDENT_LEVEL
-            names, values = \
-                _collect_parameters(ordered_parameters, *args, **kwargs)
+            names, values = _collect_parameters(ordered_parameters, *args, **kwargs)
             adatas = _collect_adatas(values)
             new_adatas: List[AnnData] = []
 
@@ -310,12 +340,11 @@ def logged(**kwargs: Callable[[Any], Any]) -> Callable[[CALLABLE], CALLABLE]:
 
                 for adata in adatas:
                     IS_TOP_LEVEL = False
-                    if hasattr(adata, '__is_top_level__'):
-                        IS_TOP_LEVEL = \
-                            IS_TOP_LEVEL or getattr(adata, '__is_top_level__')
+                    if hasattr(adata, "__is_top_level__"):
+                        IS_TOP_LEVEL = IS_TOP_LEVEL or getattr(adata, "__is_top_level__")
                     else:
                         IS_TOP_LEVEL = IS_TOP_LEVEL or CALL_LEVEL == 0
-                        setattr(adata, '__is_top_level__', CALL_LEVEL == 0)
+                        setattr(adata, "__is_top_level__", CALL_LEVEL == 0)
                         new_adatas.append(adata)
 
                 if IS_TOP_LEVEL:
@@ -326,23 +355,20 @@ def logged(**kwargs: Callable[[Any], Any]) -> Callable[[CALLABLE], CALLABLE]:
                     param_level = DEBUG
 
                 name = function.__qualname__
-                if name[0] == '_':
+                if name[0] == "_":
                     name = name[1:]
                 if logger().isEnabledFor(step_level):
-                    logger().log(step_level, '%scall %s:',
-                                 INDENT_SPACES[:2 * INDENT_LEVEL],
-                                 name)
+                    logger().log(step_level, "%scall %s:", INDENT_SPACES[: 2 * INDENT_LEVEL], name)
                     INDENT_LEVEL += 1
                 CALL_LEVEL += 1
 
                 if logger().isEnabledFor(param_level):
                     for name, value in zip(names, values):
-                        log_value = _format_value(value, name,
-                                                  formatter_by_name.get(name))
+                        log_value = _format_value(value, name, formatter_by_name.get(name))
                         if log_value is not None:
-                            logger().log(param_level, '%swith %s: %s',
-                                         INDENT_SPACES[:2 * INDENT_LEVEL],
-                                         name, log_value)
+                            logger().log(
+                                param_level, "%swith %s: %s", INDENT_SPACES[: 2 * INDENT_LEVEL], name, log_value
+                            )
 
                 return function(*args, **kwargs)
 
@@ -352,18 +378,14 @@ def logged(**kwargs: Callable[[Any], Any]) -> Callable[[CALLABLE], CALLABLE]:
                 CALL_LEVEL -= 1
                 IS_TOP_LEVEL = old_is_top_level
                 for adata in new_adatas:
-                    delattr(adata, '__is_top_level__')
+                    delattr(adata, "__is_top_level__")
 
         return wrapper
 
     return wrap  # type: ignore
 
 
-def _collect_parameters(
-    parameters: List[Parameter],
-    *args: Any,
-    **kwargs: Any
-) -> Tuple[List[str], List[Any]]:
+def _collect_parameters(parameters: List[Parameter], *args: Any, **kwargs: Any) -> Tuple[List[str], List[Any]]:
     names: List[str] = []
     values: List[Any] = []
 
@@ -371,7 +393,7 @@ def _collect_parameters(
         names.append(parameter.name)
         values.append(value)
 
-    for parameter in parameters[len(args):]:
+    for parameter in parameters[len(args) :]:
         names.append(parameter.name)
         values.append(kwargs.get(parameter.name, parameter.default))
 
@@ -389,18 +411,16 @@ def _collect_adatas(values: List[Any]) -> List[AnnData]:
 
 
 def _format_value(  # pylint: disable=too-many-return-statements,too-many-branches
-    value: Any,
-    name: str,
-    formatter: Optional[Callable[[Any], Any]] = None
+    value: Any, name: str, formatter: Optional[Callable[[Any], Any]] = None
 ) -> Optional[str]:
     if isinstance(value, Parameter.empty.__class__):
         return None
 
-    checksum = ''
-#   if utt.is_1d(value) and 'U' not in str(value.dtype) and 'o' not in str(value.dtype):
-#       checksum = ' checksum: %.20e' % utt.shaped_checksum(value)
-#   elif utt.is_2d(value):
-#       checksum = ' checksum: %.20e' % utt.shaped_checksum(value)
+    checksum = ""
+    #   if utt.is_1d(value) and 'U' not in str(value.dtype) and 'o' not in str(value.dtype):
+    #       checksum = ' checksum: %.20e' % utt.shaped_checksum(value)
+    #   elif utt.is_2d(value):
+    #       checksum = ' checksum: %.20e' % utt.shaped_checksum(value)
 
     if formatter is not None:
         value = formatter(value)
@@ -408,124 +428,108 @@ def _format_value(  # pylint: disable=too-many-return-statements,too-many-branch
             return None
 
     if isinstance(value, AnnData):
-        name = value.uns.get('__name__', 'unnamed')
+        name = value.uns.get("__name__", "unnamed")
 
-        if hasattr(value.X, 'dtype'):
-            dtype = getattr(value.X, 'dtype')
+        if hasattr(value.X, "dtype"):
+            dtype = getattr(value.X, "dtype")
         else:
-            dtype = 'unknown'
+            dtype = "unknown"
 
-        return f'{name} annotated data with {value.shape[0]} X {value.shape[1]} {dtype}s' + checksum
+        return f"{name} annotated data with {value.shape[0]} X {value.shape[1]} {dtype}s" + checksum
 
     if isinstance(value, (np.bool_, bool, str, None.__class__)):
         return str(value) + checksum
 
     if isinstance(value, (int, float, np.float32, np.float64, np.int32, np.int64)):
         value = float(value)
-        if 'random_seed' in name:
+        if "random_seed" in name:
             if value == 0:
-                return '0 (time)'
-            return f'{value:g} (reproducible)'
-        if 'rank' in name:
-            return f'{value:g}'
-        if 'fold' in name:
+                return "0 (time)"
+            return f"{value:g} (reproducible)"
+        if "rank" in name:
+            return f"{value:g}"
+        if "fold" in name:
             return fold_description(value)
-        if 'fraction' in name or 'quantile' in name:
+        if "fraction" in name or "quantile" in name:
             return fraction_description(value)
-        if 'factor' in name:
-            return f'X {value:.4g}'
-        return f'{value:g}' + checksum
+        if "factor" in name:
+            return f"X {value:.4g}"
+        return f"{value:g}" + checksum
 
-    if hasattr(value, '__qualname__'):
-        return getattr(value, '__qualname__') + checksum
+    if hasattr(value, "__qualname__"):
+        return getattr(value, "__qualname__") + checksum
 
-    if isinstance(value, (pd.Series, np.ndarray)) and value.ndim == 1 and value.dtype == 'bool':
+    if isinstance(value, (pd.Series, np.ndarray)) and value.ndim == 1 and value.dtype == "bool":
         return mask_description(value) + checksum
 
-    if hasattr(value, 'ndim'):
+    if hasattr(value, "ndim"):
         if value.ndim == 2:
-            text = f'{value.__class__.__name__} {value.shape[0]} X {value.shape[1]} {utt.matrix_dtype(value)}s'
+            text = f"{value.__class__.__name__} {value.shape[0]} X {value.shape[1]} {utt.matrix_dtype(value)}s"
             return text + checksum
 
         if value.ndim == 1:
             value = utt.to_numpy_vector(value)
             if len(value) > 100:
-                text = f'{len(value)} {value.dtype}s'
+                text = f"{len(value)} {value.dtype}s"
                 return text + checksum
             value = list(value)
 
     if isinstance(value, list):
         if len(value) > 100:
-            return f'{len(value)} {value[0].__class__.__name__}s' + checksum
-        texts = [element.uns.get('__name__', 'unnamed')
-                 if isinstance(element, AnnData) else str(element)
-                 for element in value]
+            return f"{len(value)} {value[0].__class__.__name__}s" + checksum
+        texts = [
+            element.uns.get("__name__", "unnamed") if isinstance(element, AnnData) else str(element)
+            for element in value
+        ]
         return f'[ {", ".join(texts)} ]' + checksum
 
-    raise RuntimeError(  #
-        f'unknown parameter type: {value.__class__} value: {value}')
+    raise RuntimeError(f"unknown parameter type: {value.__class__} value: {value}")  #
 
 
 def top_level(adata: AnnData) -> None:
-    '''
+    """
     Indicate that the annotated data will be returned to the top-level caller, increasing its
     logging level.
-    '''
-    setattr(adata, '__is_top_level__', True)
+    """
+    setattr(adata, "__is_top_level__", True)
 
 
-def log_return(
-    name: str,
-    value: Any,
-    *,
-    formatter: Optional[Callable[[Any], Any]] = None
-) -> bool:
-    '''
+def log_return(name: str, value: Any, *, formatter: Optional[Callable[[Any], Any]] = None) -> bool:
+    """
     Log a ``value`` returned from a function with some ``name``.
 
     If ``formatter`` is specified, use it to override the default logged value formatting.
-    '''
+    """
     if CALL_LEVEL == 0:
         top_log_level = INFO
     else:
         top_log_level = CALC
 
-    return _log_value(name, value, 'return', None, top_log_level, formatter)
+    return _log_value(name, value, "return", None, top_log_level, formatter)
 
 
 def logging_calc() -> bool:
-    '''
+    """
     Whether we are actually logging the intermediate calculations.
-    '''
-    return (IS_TOP_LEVEL and logger().isEnabledFor(CALC)) \
-        or (not IS_TOP_LEVEL and logger().isEnabledFor(DEBUG))
+    """
+    return (IS_TOP_LEVEL and logger().isEnabledFor(CALC)) or (not IS_TOP_LEVEL and logger().isEnabledFor(DEBUG))
 
 
-def log_calc(
-    name: str,
-    value: Any = None,
-    *,
-    formatter: Optional[Callable[[Any], Any]] = None
-) -> bool:
-    '''
+def log_calc(name: str, value: Any = None, *, formatter: Optional[Callable[[Any], Any]] = None) -> bool:
+    """
     Log an intermediate calculated ``value`` computed from a function with some ``name``.
 
     If ``formatter`` is specified, use it to override the default logged value formatting.
-    '''
-    return _log_value(name, value, 'calc', None, CALC, formatter)
+    """
+    return _log_value(name, value, "calc", None, CALC, formatter)
 
 
 @contextmanager
-def log_step(
-    name: str,
-    value: Any = None,
-    *,
-    formatter: Optional[Callable[[Any], Any]] = None
-) -> Iterator[None]:
-    '''
+def log_step(name: str, value: Any = None, *, formatter: Optional[Callable[[Any], Any]] = None) -> Iterator[None]:
+    """
     Same as :py:func:`log_calc`, but also further indent all the log messages inside the ``with``
     statement body.
-    '''
+    """
     global INDENT_LEVEL
 
     if log_calc(name, value, formatter=formatter):
@@ -541,117 +545,95 @@ def log_step(
 
 
 #: Convert ``per`` to the name of the ``AnnData`` data member holding it.
-MEMBER_OF_PER = \
-    dict(m='uns',
-         o='obs', v='var',
-         oo='obsp', vv='varp',
-         oa='obsm', va='varm',
-         vo='layers')
+MEMBER_OF_PER = dict(m="uns", o="obs", v="var", oo="obsp", vv="varp", oa="obsm", va="varm", vo="layers")
 
 
-def incremental(
-    adata: AnnData,
-    per: str,
-    name: str,
-    formatter: Optional[Callable[[Any], Any]] = None
-) -> None:
-    '''
+def incremental(adata: AnnData, per: str, name: str, formatter: Optional[Callable[[Any], Any]] = None) -> None:
+    """
     Declare that the named annotation will be built incrementally - set and then repeatedly modified.
-    '''
-    assert per in ('m', 'v', 'o', 'vv', 'oo', 'vo', 'va', 'oa')
-    if not hasattr(adata, '__incremental__'):
-        setattr(adata, '__incremental__', {})
-    by_name: Dict[str, Tuple[str, Optional[Callable[[Any], Any]]]] = \
-        getattr(adata, '__incremental__')
+    """
+    assert per in ("m", "v", "o", "vv", "oo", "vo", "va", "oa")
+    if not hasattr(adata, "__incremental__"):
+        setattr(adata, "__incremental__", {})
+    by_name: Dict[str, Tuple[str, Optional[Callable[[Any], Any]]]] = getattr(adata, "__incremental__")
     assert name not in by_name
     by_name[name] = (per, formatter)
 
 
 def done_incrementals(adata: AnnData) -> None:
-    '''
+    """
     Declare that all the incremental values have been fully computed.
-    '''
-    assert hasattr(adata, '__incremental__')
-    by_name: Dict[str, Tuple[str, Optional[Callable[[Any], Any]]]] = \
-        getattr(adata, '__incremental__')
-    setattr(adata, '__incremental__', [])
+    """
+    assert hasattr(adata, "__incremental__")
+    by_name: Dict[str, Tuple[str, Optional[Callable[[Any], Any]]]] = getattr(adata, "__incremental__")
+    setattr(adata, "__incremental__", [])
 
     for name, (per, formatter) in by_name.items():
-        if name == '__x__':
+        if name == "__x__":
             value = adata.X
         else:
             annotation = getattr(adata, MEMBER_OF_PER[per])
             if name not in annotation:
-                raise RuntimeError(f'missing the incremental data: {name}')
+                raise RuntimeError(f"missing the incremental data: {name}")
             value = annotation[name]
         log_set(adata, per, name, value, formatter=formatter)
 
 
 def cancel_incrementals(adata: AnnData) -> None:
-    '''
+    """
     Cancel tracking incremental annotations.
-    '''
-    assert hasattr(adata, '__incremental__')
-    delattr(adata, '__incremental__')
+    """
+    assert hasattr(adata, "__incremental__")
+    delattr(adata, "__incremental__")
 
 
 def log_set(
-    adata: AnnData,
-    per: str,
-    name: str,
-    value: Any,
-    *,
-    formatter: Optional[Callable[[Any], Any]] = None
+    adata: AnnData, per: str, name: str, value: Any, *, formatter: Optional[Callable[[Any], Any]] = None
 ) -> bool:
-    '''
+    """
     Log setting some annotated data.
-    '''
-    assert per in ('m', 'v', 'o', 'vv', 'oo', 'vo', 'va', 'oa')
+    """
+    assert per in ("m", "v", "o", "vv", "oo", "vo", "va", "oa")
 
-    is_top_level = \
-        hasattr(adata, '__is_top_level__') \
-        and getattr(adata, '__is_top_level__') \
-        and (not hasattr(adata, '__incremental__')
-             or not name in getattr(adata, '__incremental__'))
+    is_top_level = (
+        hasattr(adata, "__is_top_level__")
+        and getattr(adata, "__is_top_level__")
+        and (not hasattr(adata, "__incremental__") or name not in getattr(adata, "__incremental__"))
+    )
 
-    adata_name = adata.uns.get('__name__', 'unnamed')
-    if name == '__x__':
-        name = f'{adata_name}.X'
+    adata_name = adata.uns.get("__name__", "unnamed")
+    if name == "__x__":
+        name = f"{adata_name}.X"
     else:
         member_name = MEMBER_OF_PER[per]
-        name = f'{adata_name}.{member_name}[{name}]'
-    return _log_value(name, value, 'set', is_top_level, INFO, formatter)
+        name = f"{adata_name}.{member_name}[{name}]"
+    return _log_value(name, value, "set", is_top_level, INFO, formatter)
 
 
 def log_get(
-    adata: AnnData,
-    per: str,
-    name: Any,
-    value: Any,
-    *,
-    formatter: Optional[Callable[[Any], Any]] = None
+    adata: AnnData, per: str, name: Any, value: Any, *, formatter: Optional[Callable[[Any], Any]] = None
 ) -> bool:
-    '''
+    """
     Log getting some annotated data.
-    '''
-    assert per in ('m', 'v', 'o', 'vv', 'oo', 'vo', 'va', 'oa')
+    """
+    assert per in ("m", "v", "o", "vv", "oo", "vo", "va", "oa")
 
-    is_top_level = \
-        hasattr(adata, '__is_top_level__') \
-        and getattr(adata, '__is_top_level__') \
-        and (not hasattr(adata, '__incremental__')
-             or not name in getattr(adata, '__incremental__'))
+    is_top_level = (
+        hasattr(adata, "__is_top_level__")
+        and getattr(adata, "__is_top_level__")
+        and (not hasattr(adata, "__incremental__") or name not in getattr(adata, "__incremental__"))
+    )
 
-    adata_name = adata.uns.get('__name__', 'unnamed')
-    if isinstance(name, str) and name == '__x__':
-        name = f'{adata_name}.X'
+    adata_name = adata.uns.get("__name__", "unnamed")
+    if isinstance(name, str) and name == "__x__":
+        name = f"{adata_name}.X"
     else:
         member_name = MEMBER_OF_PER[per]
         if not isinstance(name, str):
-            name = '<data>'
-        name = f'{adata_name}.{member_name}[{name}]'
+            name = "<data>"
+        name = f"{adata_name}.{member_name}[{name}]"
 
-    return _log_value(name, value, 'get', is_top_level, CALC, formatter)
+    return _log_value(name, value, "get", is_top_level, CALC, formatter)
 
 
 def _log_value(
@@ -660,7 +642,7 @@ def _log_value(
     kind: str,
     is_top_level: Optional[bool],
     top_log_level: int,
-    formatter: Optional[Callable[[Any], Any]] = None
+    formatter: Optional[Callable[[Any], Any]] = None,
 ) -> bool:
     if is_top_level is None:
         is_top_level = IS_TOP_LEVEL
@@ -674,52 +656,50 @@ def _log_value(
         return False
 
     if value is None:
-        logger().log(level, '%s%s', INDENT_SPACES[:2 * INDENT_LEVEL], name)
+        logger().log(level, "%s%s", INDENT_SPACES[: 2 * INDENT_LEVEL], name)
     else:
         log_value = _format_value(value, name, formatter)
-        if name[0] == '-':
-            logger().log(level, '%s%s: %s',
-                         INDENT_SPACES[:2 * INDENT_LEVEL], name, log_value)
+        if name[0] == "-":
+            logger().log(level, "%s%s: %s", INDENT_SPACES[: 2 * INDENT_LEVEL], name, log_value)
         else:
-            logger().log(level, '%s%s %s: %s',
-                         INDENT_SPACES[:2 * INDENT_LEVEL], kind, name, log_value)
+            logger().log(level, "%s%s %s: %s", INDENT_SPACES[: 2 * INDENT_LEVEL], kind, name, log_value)
 
     return True
 
 
 def sizes_description(sizes: Union[utt.Vector, str]) -> str:
-    '''
+    """
     Return a string for logging an array of sizes.
-    '''
+    """
     if isinstance(sizes, str):
         return sizes
 
     sizes = utt.to_numpy_vector(sizes)
     mean = np.mean(sizes)
-    return f'{len(sizes)} {sizes.dtype}s with mean {mean:.4g}'
+    return f"{len(sizes)} {sizes.dtype}s with mean {mean:.4g}"
 
 
 def fractions_description(sizes: Union[utt.Vector, str]) -> str:
-    '''
+    """
     Return a string for logging an array of fractions (between zero and one).
-    '''
+    """
     if isinstance(sizes, str):
         return sizes
 
     sizes = utt.to_numpy_vector(sizes)
     mean = np.mean(sizes)
     percent = mean * 100
-    return f'{len(sizes)} {sizes.dtype}s with mean {mean:.4g} ({percent:.4g}%)'
+    return f"{len(sizes)} {sizes.dtype}s with mean {mean:.4g} ({percent:.4g}%)"
 
 
 def groups_description(groups: Union[utt.Vector, str]) -> str:
-    '''
+    """
     Return a string for logging an array of group indices.
 
     .. note::
 
         This assumes that the indices are consecutive, with negative values indicating "outliers".
-    '''
+    """
     if isinstance(groups, str):
         return groups
 
@@ -729,33 +709,32 @@ def groups_description(groups: Union[utt.Vector, str]) -> str:
 
     if groups_count == 0:
         assert outliers_count == len(groups)
-        return f'{len(groups)} {groups.dtype} elements with all outliers (100%)'
+        return f"{len(groups)} {groups.dtype} elements with all outliers (100%)"
 
     mean = (len(groups) - outliers_count) / groups_count
-    return ratio_description(len(groups), f'{groups.dtype} element',
-                             outliers_count, 'outliers') + \
-        f' with {groups_count} groups with mean size {mean:.4g}'
+    return (
+        ratio_description(len(groups), f"{groups.dtype} element", outliers_count, "outliers")
+        + f" with {groups_count} groups with mean size {mean:.4g}"
+    )
 
 
 def mask_description(mask: Union[str, utt.Vector]) -> str:
-    '''
+    """
     Return a string for logging a boolean mask.
-    '''
+    """
     if isinstance(mask, str):
         return mask
 
     mask = utt.to_numpy_vector(mask)
-    if mask.dtype == 'bool':
-        return ratio_description(mask.size, 'bool',
-                                 np.sum(mask > 0), 'true')
-    return ratio_description(mask.size, str(mask.dtype),
-                             np.sum(mask > 0), 'positive')
+    if mask.dtype == "bool":
+        return ratio_description(mask.size, "bool", np.sum(mask > 0), "true")
+    return ratio_description(mask.size, str(mask.dtype), np.sum(mask > 0), "positive")
 
 
 def ratio_description(denominator: float, element: str, numerator: float, condition: str) -> str:
-    '''
+    """
     Return a string for describing a ratio (including a percent representation).
-    '''
+    """
     assert numerator >= 0
     assert denominator > 0
 
@@ -765,42 +744,42 @@ def ratio_description(denominator: float, element: str, numerator: float, condit
         denominator = int(denominator)
 
     percent = (numerator * 100) / denominator
-    return f'{numerator} {condition} ({percent:.4g}%) out of {denominator} {element}s'
+    return f"{numerator} {condition} ({percent:.4g}%) out of {denominator} {element}s"
 
 
 def progress_description(amount: int, index: int, element: str) -> str:
-    '''
+    """
     Return a string for describing progress in a loop.
-    '''
+    """
     assert amount > 0
     assert 0 <= index < amount
 
     percent = ((index + 1) * 100) / amount
-    return f'#{index} out of {amount} {element}s ({percent:.4g}%)'
+    return f"#{index} out of {amount} {element}s ({percent:.4g}%)"
 
 
 def fraction_description(fraction: Optional[float]) -> str:
-    '''
+    """
     Return a string for describing a fraction (including a percent representation).
-    '''
+    """
     if fraction is None:
-        return 'None'
+        return "None"
     percent = fraction * 100
-    return f'{fraction:.4g} ({percent:.4g}%)'
+    return f"{fraction:.4g} ({percent:.4g}%)"
 
 
 def fold_description(fold: float) -> str:
-    '''
+    """
     Return a string for describing a fraction (including a percent representation).
-    '''
+    """
     if fold is None:
-        return 'None'
+        return "None"
 
     if fold >= 0:
         value = 2 ** fold
-        char = 'X'
+        char = "X"
     else:
         value = 2 ** -fold
-        char = '/'
+        char = "/"
 
-    return f'{fold:.4g} ({char} {value:.4g})'
+    return f"{fold:.4g} ({char} {value:.4g})"
